#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include <vte/vte.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "rpmutils.h"
#include "gyum_pkglist.h"
#include "gyum_utils.h"
#include "gyum_log.h"
#include "gyum_config.h"
#include "gyum_appbar.h"
#include "gyum_preferences.h"
#include "gyum_cache.h"
#include "gyum_rpm.h"
#include "yumrepo.h"
#include "rpminfo.h"

static PkgListFilter status=PKG_LIST_FILTER_ALL;
static PkgListFilter provider=0;
static PkgListSearch search=PKG_LIST_SEARCH_PACKAGE_NAME;
static GtkWidget *gyum=NULL;
static gboolean set_filter_default=TRUE;
static gboolean invert=FALSE;

void
on_gyum_realize                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *search;
	gchar *str;
	gboolean b;
	GtkWidget *cbx;
	GtkWidget *entry;
	if (strcmp(getenv("USER"),"root"))
	{
		GtkWidget *msg;
		gtk_widget_set_sensitive(lookup_widget(widget,"mpreferences"),FALSE);
		gtk_widget_set_sensitive(lookup_widget(widget,"hbox_cache"),FALSE);
		msg=gtk_message_dialog_new(GTK_WINDOW(widget),
			GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,
			GTK_BUTTONS_OK,
			_("You aren't privileged user (root).\nYou can view packages only..."));
		splash_hide();
		gtk_dialog_run(GTK_DIALOG(msg));
		gtk_widget_destroy(msg);
	}
	yumutils_init();
	gyum=widget;
	gyum_appbar_init(lookup_widget(widget,"appbar1"));
	gyum_create_terminal(widget);
	gyum_cache_init(widget);
	gyum_yum_log_init(widget);
	
	search=lookup_widget(widget,"search");
	gyum_pkglist_init(lookup_widget(widget,"package_list"));
	cbx=lookup_widget(widget,"search");
	entry=gtk_bin_get_child(GTK_BIN(cbx));
	g_signal_connect ((gpointer) gtk_bin_get_child(GTK_BIN(search)), "key-press-event",
                    G_CALLBACK (on_search_entry_key_press_event),
                    NULL);
	gtk_combo_box_set_active(GTK_COMBO_BOX(lookup_widget(widget,"cbx_search_in")),1);
	
	str=gyum_config_get_string("default_filter");
	if (str)
	{
		gchar *wn=g_strdup_printf("rtb_%s",str);
		GtkWidget *w=lookup_widget(widget,wn);
		if (w)
		{
			set_filter_default=TRUE;
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(w),TRUE);
			set_filter_default=FALSE;
		}
	}
	g_free(str);
	set_filter_default=TRUE;
	b=gyum_config_get_boolean("distributor_packages");
	{
		GtkWidget *ttb_distributor_packages=lookup_widget(widget,"ttb_distributor_packages");
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ttb_distributor_packages),b);
	}
	b=gyum_config_get_boolean("external_packages");
	{
		GtkWidget *ttb_external_packages=lookup_widget(widget,"ttb_external_packages");
		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ttb_external_packages),b);
	}
	set_filter_default=FALSE;
	
	if (gyum_config_get_boolean("startup_loading"))
	{
		gyum_pkglist_load();
	}

	set_filter_default=TRUE;
	gtk_entry_set_text(GTK_ENTRY(entry),"*");
	set_filter_default=FALSE;
	gyum_cache_fill(yumrepo_get_cache_root());
}


void package_informations_concat(GtkWidget *widget)
{
	GtkWidget *vbox_installed_package;
	GtkWidget *vbox_available_package;
	GtkWidget *frame_installed_package;
	GtkWidget *frame_available_package;
	GtkWidget *notebook_package;
	vbox_available_package=lookup_widget(widget,"vbox_available_package");
	vbox_installed_package=lookup_widget(widget,"vbox_installed_package");
	frame_installed_package=lookup_widget(widget,"frame_installed_package");
	gtk_widget_show(vbox_installed_package);
	gtk_widget_reparent(frame_installed_package,vbox_installed_package);
	frame_available_package=lookup_widget(widget,"frame_available_package");
	gtk_widget_show(vbox_available_package);
	gtk_widget_reparent(frame_available_package,vbox_available_package);
	notebook_package=lookup_widget(widget,"notebook_package");
	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook_package),0);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook_package),FALSE);
}


void package_informations_split(GtkWidget *widget)
{
	GtkWidget *vbox_installed_package;
	GtkWidget *vbox_available_package;
	GtkWidget *vbox_general_information;
	GtkWidget *nbp_vbox_installed_package;
	GtkWidget *nbp_vbox_available_package;
	GtkWidget *frame_installed_package;
	GtkWidget *frame_available_package;
	GtkWidget *notebook_package;
	vbox_general_information=lookup_widget(widget,"vbox_general_information");
	vbox_available_package=lookup_widget(widget,"vbox_available_package");
	vbox_installed_package=lookup_widget(widget,"vbox_installed_package");
	
	nbp_vbox_installed_package=lookup_widget(widget,"nbp_vbox_installed_package");
	frame_installed_package=lookup_widget(widget,"frame_installed_package");
	gtk_widget_reparent(frame_installed_package,nbp_vbox_installed_package);
	gtk_widget_hide(vbox_installed_package);
		
	nbp_vbox_available_package=lookup_widget(widget,"nbp_vbox_available_package");
	frame_available_package=lookup_widget(widget,"frame_available_package");
	gtk_widget_reparent(frame_available_package,nbp_vbox_available_package);
	gtk_widget_hide(vbox_available_package);

	notebook_package=lookup_widget(widget,"notebook_package");
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook_package),TRUE);
}


void
on_gyum_size_allocate                  (GtkWidget       *widget,
                                        GdkRectangle    *allocation,
                                        gpointer         user_data)
{
	if (!gyum_config_get_boolean("gui_show_notebook"))
	{
		if (allocation->height>740)
		{
			package_informations_concat(widget);
		}
		else
		{
			package_informations_split(widget);
		}
	}
}


void
on_mquit_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *gyum=lookup_widget(GTK_WIDGET(menuitem),"gyum");
	if (!on_gyum_delete_event(gyum,NULL,NULL)) gtk_widget_destroy(gyum);
}


void
on_mrefresh_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gyum_pkglist_load();
}


void
on_mcheck_updates_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gyum_pkglist_check_updates();
}


static gchar *old_cachedir=NULL;
static gchar *old_logfile=NULL;
static gchar *old_yum_repos_d=NULL;

void
on_mpreferences_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!gyum_preferences_active())
	{
		GtkWidget *gyum_preferences;
		old_cachedir=g_strdup(yumrepo_get_cache_root());
		old_logfile=g_strdup(yumrepo_get_log_file());
		old_yum_repos_d=g_strdup(yumrepo_get_yum_repos_d());
		gyum_preferences=create_gyum_preferences();
		gtk_widget_show(gyum_preferences);
	}
}


void
on_mabout_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *gyum_about=create_gyum_about();
	gtk_widget_show(gyum_about);
}

static gboolean tool_status_change=FALSE;

void
on_mall_packages_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"rtb_all")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_minstalled_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"rtb_installed")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_mnot_installed_packages_activate    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"rtb_not_installed")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_mupdates_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"rtb_updates")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_mmajor_change_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"rtb_major_change")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_mminor_change_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"rtb_minor_change")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_mmini_change_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"rtb_mini_change")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_mdistributor_packages_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"ttb_distributor_packages")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}


void
on_mexternal_packages_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!tool_status_change)
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(lookup_widget(GTK_WIDGET(menuitem),"ttb_external_packages")),
		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)));
}

void
on_tb_refresh_clicked                  (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gyum_pkglist_load();
}


void
on_tb_check_updates_clicked            (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gyum_pkglist_check_updates();
}


void
on_rtb_all_toggled                     (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mall_packages")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status=PKG_LIST_FILTER_ALL,provider,invert,set_filter_default);
}


void
on_rtb_installed_toggled           (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"minstalled")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status=PKG_LIST_FILTER_INSTALLED,provider,invert,set_filter_default);
}


void
on_rtb_not_installed_toggled           (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mnot_installed_packages")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status=PKG_LIST_FILTER_NOT_INSTALLED,provider,invert,set_filter_default);
}


void
on_rtb_updates_toggled                 (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mupdates")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status=PKG_LIST_FILTER_UPDATES,provider,invert,set_filter_default);
}


void
on_rtb_major_change_toggled            (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mmajor_change")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status=PKG_LIST_FILTER_MAJOR_CHANGE,provider,invert,set_filter_default);
}


void
on_rtb_minor_change_toggled            (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mminor_change")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status=PKG_LIST_FILTER_MINOR_CHANGE,provider,invert,set_filter_default);
}


void
on_rtb_mini_change_toggled             (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mmini_change")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status=PKG_LIST_FILTER_MINI_CHANGE,provider,invert,set_filter_default);
}


void
on_ttb_distributor_packages_toggled    (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mdistributor_packages")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status,provider^=PKG_LIST_FILTER_DISTRIBUTOR,invert,set_filter_default);
}


void
on_ttb_external_packages_toggled       (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	tool_status_change=TRUE;
	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(toggletoolbutton),"mexternal_packages")),
		gtk_toggle_tool_button_get_active(toggletoolbutton));
	tool_status_change=FALSE;
	gyum_pkglist_filter(status,provider^=PKG_LIST_FILTER_OTHER,invert,set_filter_default);
}

void command_execute_done(pid_t pid,gint status,GtkWidget *widget)
{
	gyum_controllers_set_sensitive(TRUE);
	gyum_cache_cleaner_set_sensitive(TRUE);
	gyum_appbar_progress_end();
	gyum_pkglist_command_end();
	if (!status)
	{
		yumutils_file_remove(".last");
		gyum_pkglist_load();
	}
	gyum_cache_reinit();
	gyum_cache_fill(yumrepo_get_cache_root());
}

static PkgListOperation last_operation;

void command_execute(PkgListOperation operation)
{
	gchar *cmd;
	GtkWidget *gyum_warning;
	GtkWidget *textview;
	GtkTextBuffer *textbuffer;
	last_operation=operation;
	cmd=gyum_pkglist_make_command(NULL,operation,FALSE);
	gyum_warning=create_gyum_warning();
	textview=lookup_widget(gyum_warning,"gyum_warning_command");
	textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	gtk_text_buffer_set_text(textbuffer,cmd,-1);
	switch (gtk_dialog_run(GTK_DIALOG(gyum_warning)))
	{
		case GTK_RESPONSE_OK:
			{
				gchar *edited_cmd;
				GtkTextIter start,end;
				const gchar *msg;
				switch (operation)
				{
					case PKG_LIST_OPERATION_INSTALL:
						msg=_("Executing install");
						break;
					case PKG_LIST_OPERATION_UPDATE:
						msg=_("Executing update");
						break;
					case PKG_LIST_OPERATION_REMOVE:
						msg=_("Executing remove");
						break;
					case PKG_LIST_OPERATION_NONE:
						break;
				}
				gtk_text_buffer_get_start_iter(textbuffer,&start);
				gtk_text_buffer_get_end_iter(textbuffer,&end);
				edited_cmd=gtk_text_buffer_get_text(textbuffer,&start,&end,FALSE);
				gtk_widget_destroy(gyum_warning);
				gtk_notebook_set_current_page(
					GTK_NOTEBOOK(lookup_widget(gyum,"gyum_notebook")),1);
				gyum_controllers_set_sensitive(FALSE);
				gyum_cache_cleaner_set_sensitive(FALSE);
				yumutils_file_save(".last");
				gyum_appbar_progress_begin();
				gyum_pkglist_command_begin();
				gyum_command_execute(
					msg,
					edited_cmd,TRUE,
					(GECFunc)command_execute_done,NULL);
				g_free(edited_cmd);
			}
			break;
		case GTK_RESPONSE_CANCEL:
			gtk_widget_destroy(gyum_warning);
			break;
	}
	g_free(cmd);
}

void
on_bt_op_install_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	command_execute(PKG_LIST_OPERATION_INSTALL);
}


void
on_bt_op_update_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	command_execute(PKG_LIST_OPERATION_UPDATE);
}


void
on_bt_op_remove_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	command_execute(PKG_LIST_OPERATION_REMOVE);
}


void
on_package_list_cursor_changed         (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	gyum_pkglist_cursor_changed();
}


void
on_rb_op_none_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation(PKG_LIST_OPERATION_NONE);
}


void
on_rb_op_install_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation(PKG_LIST_OPERATION_INSTALL);
}


void
on_rb_op_update_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation(PKG_LIST_OPERATION_UPDATE);
}


void
on_rb_op_remove_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation(PKG_LIST_OPERATION_REMOVE);
}


gboolean find;

gboolean search_combobox_item(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,const gchar *text)
{
	gchar *str;
	gtk_tree_model_get(model,iter,0,&str,-1);
	find=!strcmp(text,str);
	g_free(str);
	return find;
}

void search_text(GtkWidget *widget)
{
	GtkWidget *cbx=lookup_widget(widget,"search");
	GtkWidget *entry=gtk_bin_get_child(GTK_BIN(cbx));
	const gchar *text=gtk_entry_get_text(GTK_ENTRY(entry));
	if (text)
	{
		GtkTreeModel *model;
		model=gtk_combo_box_get_model(GTK_COMBO_BOX(cbx));
		find=FALSE;
		gtk_tree_model_foreach(model,(GtkTreeModelForeachFunc)search_combobox_item,(gpointer)text);
		if (!find)
			gtk_combo_box_prepend_text(GTK_COMBO_BOX(cbx),text);
		gyum_pkglist_search(search,text);
	}
}

void
on_bt_search_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	search_text(GTK_WIDGET(button));
}


void
on_cbx_search_in_changed               (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	GtkTreeIter iter;
	if (gtk_combo_box_get_active_iter(combobox,&iter))
	{
		gchar *str=NULL;
		GtkTreeModel *model;
		model=gtk_combo_box_get_model(combobox);
		gtk_tree_model_get(model,&iter,0,&str,-1);
		if (str)
		{
			if (!strcmp(str,_("all"))) search=PKG_LIST_SEARCH_ALL;
			else
			if (!strcmp(str,_("packagename"))) search=PKG_LIST_SEARCH_PACKAGE_NAME;
			else
			if (!strcmp(str,_("repository"))) search=PKG_LIST_SEARCH_REPOSITORY;
			else
			if (!strcmp(str,_("summary & description"))) search=PKG_LIST_SEARCH_SUMMARY_N_DESCRIPTION;
			else
			if (!strcmp(str,_("selections for install"))) search=PKG_LIST_SEARCH_SELECTIONS_FOR_INSTALL;
			else
			if (!strcmp(str,_("selections for update"))) search=PKG_LIST_SEARCH_SELECTIONS_FOR_UPDATE;
			else
			if (!strcmp(str,_("selections for remove"))) search=PKG_LIST_SEARCH_SELECTIONS_FOR_REMOVE;
			g_free(str);
			//search_text(GTK_WIDGET(combobox));
		}
	}
}

gboolean
on_search_entry_key_press_event        (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	if ((event->keyval==GDK_KP_Enter) ||
		(event->keyval==GDK_Return))
	{
		search_text(widget);
		return TRUE;
	}
	return FALSE;
}

void
on_tb_information_clicked              (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *gyum_notebook;
	gint page_num;
	gyum_notebook=lookup_widget(GTK_WIDGET(toolbutton),"gyum_notebook");
	page_num=gtk_notebook_get_current_page(GTK_NOTEBOOK(gyum_notebook));
	if (!page_num)
	{
		const gchar *text;
		text=gtk_entry_get_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(toolbutton),"pkg_name")));
		rpminfo_create(text);
	}
	else
	{
		gchar *package;
		package=gyum_cache_get_current_package();
		if (package)
		{
			rpminfo_create_from_file(package);
			g_free(package);
		}
	}
}

void
on_bt_rpminfo_ok_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *window;
	window=lookup_widget(GTK_WIDGET(button),"gyum_rpminfo");
	gtk_widget_destroy(window);
}

void
on_bt_rpminfo_open_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	rpminfo_file_open(GTK_WIDGET(button));
}


void
on_bt_rpminfo_requires_show_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	rpminfo_show_requires(GTK_WIDGET(button));
}


void
on_bt_rpminfo_conflicts_show_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
	rpminfo_show_conflicts(GTK_WIDGET(button));
}

void
on_rpminfo_notebook_switch_page        (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        guint            page_num,
                                        gpointer         user_data)
{
	if (page_num==5)
		rpminfo_show_depends(GTK_WIDGET(notebook));
}


void
on_gyum_rpminfo_unrealize              (GtkWidget       *widget,
                                        gpointer         user_data)
{
	rpminfo_remove(widget);
}


gboolean
on_gyum_delete_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	if (gyum_config_get_boolean("confirm_exit"))
	{
		GtkWidget *que;
		gint ret;
		que=gtk_message_dialog_new(GTK_WINDOW(widget),
			GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
			_("Do you want to exit from the program?"));
		ret=gtk_dialog_run(GTK_DIALOG(que));
		gtk_widget_destroy(que);
		switch (ret)
		{
			case GTK_RESPONSE_NO:
				return TRUE;
		}
	}
	gyum_yum_log_unload(widget);
	return FALSE;
}


void
on_gyum_unrealize                      (GtkWidget       *widget,
                                        gpointer         user_data)
{
	rpminfo_close_all();
	gyum_cache_unload(widget);
	gyum_pkglist_free();
	gnome_config_sync();
	remove("/tmp/.gnome-yum-lock");
	gtk_exit(0);
}

void
on_mselect_all_to_install_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation_for_all(PKG_LIST_OPERATION_INSTALL);
}


void
on_mselect_all_to_update_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation_for_all(PKG_LIST_OPERATION_UPDATE);
}


void
on_mselect_all_to_remove_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation_for_all(PKG_LIST_OPERATION_REMOVE);
}


void
on_mreset_selections_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gyum_pkglist_set_operation_for_all(PKG_LIST_OPERATION_NONE);
}

void
on_bt_cache_clean_packages_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *msg=gtk_message_dialog_new(NULL,
		GTK_DIALOG_MODAL,
		GTK_MESSAGE_WARNING,
		GTK_BUTTONS_YES_NO,
		_("This operation remove all downloaded packages from the cache. Do you want to execute this operation?"));
	gint ret=gtk_dialog_run(GTK_DIALOG(msg));
	gtk_widget_destroy(msg);
	switch (ret)
	{
		case GTK_RESPONSE_YES:
			gyum_cache_clean(GYUM_CACHE_CLEAN_PACKAGES);
			break;
	}
}


void
on_bt_cache_clean_headers_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *msg=gtk_message_dialog_new(NULL,
		GTK_DIALOG_MODAL,
		GTK_MESSAGE_WARNING,
		GTK_BUTTONS_YES_NO,
		_("This operation remove information of the packages from the cache. Do you want to execute this operation?"));
	gint ret=gtk_dialog_run(GTK_DIALOG(msg));
	gtk_widget_destroy(msg);
	switch (ret)
	{
		case GTK_RESPONSE_YES:
			gyum_cache_clean(GYUM_CACHE_CLEAN_HEADERS);
			break;
	}
}


void
on_bt_cache_move_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *dir;
	gint ret;
	gchar *dest;
	dir=gtk_file_chooser_dialog_new(
			_("Select destination directory"),NULL,GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
			GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OPEN,GTK_RESPONSE_ACCEPT,NULL);
	dest=gyum_config_get_string("cache_move_destination");
	if (dest)
 		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dir),dest);
	ret=gtk_dialog_run(GTK_DIALOG(dir));
	switch (ret)
	{
		case GTK_RESPONSE_ACCEPT:
			dest=gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dir));
			gyum_config_set_string("cache_move_destination",dest);
			gtk_widget_destroy(dir);
			gdk_window_process_all_updates();
			gyum_cache_pkgs_move(dest);
			break;
		default:
			gtk_widget_destroy(dir);
			gdk_window_process_all_updates();
			break;
	}
	g_free(dest);
}


void
on_bt_cache_remove_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *msg=gtk_message_dialog_new(NULL,
		GTK_DIALOG_MODAL,
		GTK_MESSAGE_WARNING,
		GTK_BUTTONS_YES_NO,
		_("This operation remove selected packages from the cache. Do you want to execute this operation?"));
	gint ret=gtk_dialog_run(GTK_DIALOG(msg));
	gtk_widget_destroy(msg);
	gdk_window_process_all_updates();
	switch (ret)
	{
		case GTK_RESPONSE_YES:
			gyum_cache_pkgs_remove();
			break;
	}
}


void
on_gyum_preferences_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(lookup_widget(gyum,"mpreferences"),FALSE);
	gyum_preferences_init(widget);
}


void
on_gyum_preferences_unrealize          (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(lookup_widget(gyum,"mpreferences"),TRUE);
}


void
on_repo_list_cursor_changed            (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	gyum_repository_cursor_changed();
}


void
on_cache_pkglist_cursor_changed        (GtkTreeView     *treeview,
                                        gpointer         user_data)
{

}

void
on_bt_repo_new_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_repository_new();
}


void
on_bt_repo_remove_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *msg=
		gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,
			GTK_BUTTONS_YES_NO,_("Do you want remove this repository? (It will remove from disk too.)"));
	switch (gtk_dialog_run(GTK_DIALOG(msg)))
	{
		case GTK_RESPONSE_YES:
			gyum_repository_remove();
			break;
	}
	gtk_widget_destroy(msg);
}


void
on_repo_enabled_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_repository_edit("enabled",
		gtk_toggle_button_get_active(togglebutton) ? "1":"0");
}


void
on_repo_gpgcheck_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_repository_edit("gpgcheck",
		gtk_toggle_button_get_active(togglebutton) ? "1":"0");
}


void
on_repo_id_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
	g_warning("repo_id changed: %s",gtk_entry_get_text(GTK_ENTRY(editable)));
	gyum_repository_edit("id",
		gtk_entry_get_text(GTK_ENTRY(editable)));
}

void
on_repo_name_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gyum_repository_edit("name",
		gtk_entry_get_text(GTK_ENTRY(editable)));
}


void
on_repo_baseurl_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gyum_repository_edit("baseurl",
		gtk_entry_get_text(GTK_ENTRY(editable)));
}


void
on_repo_mirrorlist_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gyum_repository_edit("mirrorlist",
		gtk_entry_get_text(GTK_ENTRY(editable)));
}


void
on_repo_gpgkey_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gyum_repository_edit("gpgkey",
		gtk_entry_get_text(GTK_ENTRY(editable)));
}


void
on_ifconf_terminal_background_color_set
                                        (GtkColorButton  *colorbutton,
										 gpointer         user_data)
{
	GdkColor color;
	gtk_color_button_get_color(colorbutton,&color);
	gyum_terminal_set_color(GYUM_TERMINAL_COLOR_BACKGROUND,&color);
}


void
on_ifconf_terminal_foreground_color_set
                                        (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;
	gtk_color_button_get_color(colorbutton,&color);
	gyum_terminal_set_color(GYUM_TERMINAL_COLOR_FOREGROUND,&color);
}


void
on_ifconf_gui_font_font_set            (GtkFontButton   *fontbutton,
                                        gpointer         user_data)
{
	const gchar *fontname;
	fontname=gtk_font_button_get_font_name(fontbutton);
	gyum_preferences_set_variable("list_font",
			gyum_config_set_string_value(fontname));
}


void
on_ifconf_terminal_font_font_set       (GtkFontButton   *fontbutton,
                                        gpointer         user_data)
{
	const gchar *fontname;
	fontname=gtk_font_button_get_font_name(fontbutton);
	gyum_terminal_set_font(fontname);
}


void
on_ifconf_packageinfo_startup_loading_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_preferences_set_variable("startup_loading",
		gyum_config_set_boolean_value(gtk_toggle_button_get_active(togglebutton)));
}

void
on_ifconf_packageinfo_distributor_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_preferences_set_variable("distributor_packages",
		gyum_config_set_boolean_value(gtk_toggle_button_get_active(togglebutton)));
}


void
on_ifconf_packageinfo_external_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_preferences_set_variable("external_packages",
		gyum_config_set_boolean_value(gtk_toggle_button_get_active(togglebutton)));
}

void
on_ifconf_binary_paths_cursor_changed  (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	gyum_if_config_treeview_cursor_changed(treeview,"binary");
}


void
on_ifconf_binary_path_remove_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_if_config_treeview_item_remove(button,"binary");
}


void
on_ifconf_binary_path_add_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_if_config_treeview_item_add(button,"binary");
}


void
on_ifconf_library_paths_cursor_changed (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	gyum_if_config_treeview_cursor_changed(treeview,"library");
}


void
on_ifconf_library_path_remove_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_if_config_treeview_item_remove(button,"library");
}


void
on_ifconf_library_path_add_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_if_config_treeview_item_add(button,"library");
}

void
on_ifconf_select_binary_path_changed   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gyum_if_config_fileentry_changed(GNOME_FILE_ENTRY(editable),"binary");
}


void
on_ifconf_select_library_path_changed  (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gyum_if_config_fileentry_changed(GNOME_FILE_ENTRY(editable),"library");
}


gboolean
on_ifconf_terminal_priority_change_value
                                        (GtkRange        *range,
                                        GtkScrollType    arg1,
                                        gdouble          arg2,
                                        gpointer         user_data)
{
	if (arg2<-5.0) arg2=-5.0;
	else
	if (arg2>5.0) arg2=5.0;
	gyum_preferences_set_variable("terminal_priority",
		gyum_config_set_int_value(arg2));
	return FALSE;
}


gboolean
on_ifconf_gui_sensitivity_change_value (GtkRange        *range,
                                        GtkScrollType    arg1,
                                        gdouble          arg2,
                                        gpointer         user_data)
{
	if (arg2<100.0) arg2=100.0;
	else
	if (arg2>1000.0) arg2=1000.0;
	gyum_preferences_set_variable("gui_sensitivity",
		gyum_config_set_int_value(arg2));
	return FALSE;
}


void
on_bt_repo_import_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *gyum_repo_import=create_gyum_repo_import();
	switch (gtk_dialog_run(GTK_DIALOG(gyum_repo_import)))
	{
		case GTK_RESPONSE_OK:
			gyum_repository_import(gyum_repo_import);
			break;
	}
	gtk_widget_destroy(gyum_repo_import);
}


void
on_ifconf_gui_show_warning_toggled     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_preferences_set_variable("gui_show_warning",
		gyum_config_set_boolean_value(gtk_toggle_button_get_active(togglebutton)));
}


void
on_ifconf_gui_show_notebook_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean show=gtk_toggle_button_get_active(togglebutton);
	gyum_config_set_boolean("gui_show_notebook",show);
	if (show)
		package_informations_split(gyum);
	else
	{
		gint height;
		gtk_window_get_size(GTK_WINDOW(gyum),NULL,&height);
		if (height>740)
			package_informations_concat(gyum);
	}
}


void
on_bt_terminal_stop_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_terminal_stop();
}


void
on_cb_search_invert_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	invert=gtk_toggle_button_get_active(togglebutton);
	gyum_pkglist_filter(status,provider,invert,set_filter_default);
}


void
on_gyum_notebook_switch_page           (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        guint            page_num,
                                        gpointer         user_data)
{
	gboolean ok=TRUE;
	if (!page_num)
	{
		GtkWidget *pkg_version;
		const gchar *text;
		pkg_version=lookup_widget(GTK_WIDGET(notebook),"tb_information");
		text=gtk_entry_get_text(GTK_ENTRY(pkg_version));
		if (!text || !*text) ok=FALSE;
	}
	gtk_widget_set_sensitive(
		lookup_widget(GTK_WIDGET(notebook),"tb_information"),
		ok && (!page_num || (page_num==3)));
}

void
on_ifconf_gui_show_splash_window_toggled      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gyum_preferences_set_variable("show_splash_window",
		gyum_config_set_boolean_value(gtk_toggle_button_get_active(togglebutton)));
}


gboolean
on_gyum_splash_button_press_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	splash_hide();
	return FALSE;
}


void
on_bt_cache_execute_rpm_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_rpm_show();
}


void
on_gyum_execute_rpm_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_gyum_rpm_druidpage_command_prepare  (GnomeDruidPage  *gnomedruidpage,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
	gyum_rpm_command_page_prepare();
}


void
on_bt_gyum_rpm_file_remove_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_rpm_file_remove();
}


void
on_bt_gyum_rpm_file_add_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	gyum_rpm_file_add();
}


void
on_gyum_rpm_druidpage_finish_finish    (GnomeDruidPage  *gnomedruidpage,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
	gyum_rpm_command_execute(lookup_widget(gyum,"gyum_notebook"));
}


void
on_gyum_rpm_druid_cancel               (GnomeDruid      *gnomedruid,
                                        gpointer         user_data)
{
	GtkWidget *que;
	gint ret;
	que=gtk_message_dialog_new(NULL,0,
		GTK_MESSAGE_QUESTION,
		GTK_BUTTONS_YES_NO,
		_("Do you want to exit from the RPM command editor?"));
	ret=gtk_dialog_run(GTK_DIALOG(que));
	gtk_widget_destroy(que);
	switch (ret)
	{
		case GTK_RESPONSE_YES:
			gyum_rpm_destroy();
			break;
	}
}

void
on_gyum_execute_rpm_unrealize          (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_ifconf_gui_not_installed_color_color_set
                                        (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;
	gtk_color_button_get_color(colorbutton,&color);
	gyum_preferences_set_variable("gui_not_installed_color",
		gyum_config_set_color_value(&color));
}


void
on_ifconf_gui_installed_color_color_set
                                        (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;
	gtk_color_button_get_color(colorbutton,&color);
	gyum_preferences_set_variable("gui_installed_color",
		gyum_config_set_color_value(&color));
}


void
on_ifconf_gui_update_color_color_set   (GtkColorButton  *colorbutton,
                                        gpointer         user_data)
{
	GdkColor color;
	gtk_color_button_get_color(colorbutton,&color);
	gyum_preferences_set_variable("gui_update_color",
		gyum_config_set_color_value(&color));
}

void
on_mhelp_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_mopen_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (yumutils_file_load("save")) gyum_pkglist_fill();
}


void
on_msave_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	yumutils_file_save("save");
}


void
on_cb_disable_unused_repos_toggled     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gchar *command;
	GtkTextBuffer *textbuffer;
	command=gyum_pkglist_make_command(NULL,last_operation,
			gtk_toggle_button_get_active(togglebutton));
	textbuffer=gtk_text_view_get_buffer(
		GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(togglebutton),"gyum_warning_command")));
	gtk_text_buffer_set_text(textbuffer,command,-1);
	g_free(command);
}

void
on_rpminfo_requires_cursor_changed     (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"bt_rpminfo_requires_show"),TRUE);
}


void
on_rpminfo_conflicts_cursor_changed    (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"bt_rpminfo_conflicts_show"),TRUE);
}


void
on_rpminfo_files_cursor_changed        (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"bt_rpminfo_open"),TRUE);
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"bt_rpminfo_copy"),TRUE);
}


void
on_bt_rpminfo_copy_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	rpminfo_file_copy(GTK_WIDGET(button));
}


void
on_bt_gyum_preferences_cancel_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *gyum_preferences;
	gboolean changed;
	gyum_preferences=lookup_widget(GTK_WIDGET(button),"gyum_preferences");
	if (gyum_preferences_unload(gyum_preferences,FALSE,&changed))
	{
		gtk_widget_destroy(gyum_preferences);
		if (strcmp(old_cachedir,yumrepo_get_cache_root()))
		{
			gyum_cache_reinit();
			gyum_cache_fill(yumrepo_get_cache_root());
		}
		g_free(old_cachedir);
		if (strcmp(old_logfile,yumrepo_get_log_file())) gyum_yum_log_init(gyum);
		g_free(old_logfile);
		if (strcmp(old_yum_repos_d,yumrepo_get_yum_repos_d()) &&
			yumutils_package_list_available())
		{
			yumutils_package_list_clean();
			gyum_pkglist_load();
		}
		g_free(old_yum_repos_d);
	}
}


void
on_bt_gyum_preferences_apply_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *gyum_preferences;
	gboolean changed;
	gyum_preferences=lookup_widget(GTK_WIDGET(button),"gyum_preferences");
	if (gyum_preferences_unload(gyum_preferences,TRUE,&changed))
	{
		gtk_widget_destroy(gyum_preferences);
		if (strcmp(old_cachedir,yumrepo_get_cache_root()))
		{
			gyum_cache_reinit();
			gyum_cache_fill(yumrepo_get_cache_root());
		}
		g_free(old_cachedir);
		if (strcmp(old_logfile,yumrepo_get_log_file())) gyum_yum_log_init(gyum);
		g_free(old_logfile);
		if (strcmp(old_yum_repos_d,yumrepo_get_yum_repos_d()) &&
			yumutils_package_list_available())
		{
			yumutils_package_list_clean();
			gyum_pkglist_load();
		}
		g_free(old_yum_repos_d);
	}
}
