#include <glib/gi18n.h>
#include "gyum_appbar.h"

static gboolean running=FALSE;
static GtkWidget *gyum_appbar=NULL;
static GtkProgressBar *gyum_progress;
static gint gyum_status_counter[APPBAR_STATUS_ITEM_NUM];

void gyum_appbar_set_status(gint ap,gint ip,gint dp,gint sel,gint i,gint u,gint r)
{
	gchar *str;
	str=g_strdup_printf(
		_("available: %i       installed: %i       displayed: %i       selected: %i   (install: %i,update: %i,remove: %i)"),
		ap,ip,dp,sel,i,u,r);
	gnome_appbar_set_status(GNOME_APPBAR(gyum_appbar),str);
	g_free(str);
}

void gyum_appbar_init(GtkWidget *appbar)
{
	gyum_appbar=appbar;
	gyum_progress=gnome_appbar_get_progress(GNOME_APPBAR(gyum_appbar));
	gtk_progress_bar_set_pulse_step(gyum_progress,0.2);
	gyum_appbar_reset_status();
}

void gyum_appbar_reset_status()
{
	AppBarStatusItem i;
	for (i=APPBAR_STATUS_ITEM_AVAILABLE;i<APPBAR_STATUS_ITEM_NUM;i++) gyum_status_counter[i]=0;
	gyum_appbar_set_status(0,0,0,0,0,0,0);
}

gboolean gyum_progress_pulse(GtkProgressBar *progress)
{
	gtk_progress_bar_pulse(progress);
	return running;
}

void gyum_appbar_progress_begin()
{
	running=TRUE;
	g_timeout_add_full(G_PRIORITY_HIGH_IDLE,
			1000,
			(GSourceFunc)gyum_progress_pulse,
			gyum_progress,
			NULL);
}

void gyum_appbar_progress_end()
{
	running=FALSE;
}

void gyum_appbar_progress_set_fraction(gfloat fraction)
{
	gint percent=(fraction*100);
	if ((percent%10)==0)
	{
		gtk_progress_bar_set_fraction(gyum_progress,fraction);
	}
}

void gyum_appbar_status_increment(AppBarStatusItem status,gint add)
{
	gyum_status_counter[status]+=add;
}

gint gyum_appbar_get_status(AppBarStatusItem status)
{
	return gyum_status_counter[status];
}

void gyum_appbar_display_status()
{
	gyum_appbar_set_status(
		gyum_status_counter[APPBAR_STATUS_ITEM_AVAILABLE],
		gyum_status_counter[APPBAR_STATUS_ITEM_INSTALLED],
		gyum_status_counter[APPBAR_STATUS_ITEM_DISPLAYED],
			gyum_status_counter[APPBAR_STATUS_ITEM_SELECT_INSTALL]+
			gyum_status_counter[APPBAR_STATUS_ITEM_SELECT_UPDATE]+
			gyum_status_counter[APPBAR_STATUS_ITEM_SELECT_REMOVE],
		gyum_status_counter[APPBAR_STATUS_ITEM_SELECT_INSTALL],
		gyum_status_counter[APPBAR_STATUS_ITEM_SELECT_UPDATE],
		gyum_status_counter[APPBAR_STATUS_ITEM_SELECT_REMOVE]);
}
