#include <sys/stat.h>
#include <dirent.h>
#include <string.h>
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include "support.h"
#include "interface.h"
#include "gyum_utils.h"
#include "gyum_config.h"
#include "gyum_cache.h"
#include "yumrepo.h"
#include "yumutils.h"

enum {
	CACHE_COLUMN_SELECT,
	CACHE_COLUMN_NAME,
	CACHE_COLUMN_INSTALLED,
	CACHE_COLUMN_SIZE,
	CACHE_COLUMN_REPOSITORY,
	CACHE_COLUMN_NUM
};

static GtkWidget *cache_pkglist=NULL;
static GtkTreeModel *cache_pkglist_model=NULL;

static GdkColor not_installed_fg;
static GdkColor installed_fg;
static GdkColor update_fg;
static gchar *fontname=NULL;

void gyum_cache_row_set(GtkTreeViewColumn *column,GtkCellRenderer *renderer,GtkTreeModel *model,GtkTreeIter *iter,gpointer data)
{
	gchar *name;
	gchar *iver;
	GdkColor *fg;
	gint status=0;
	gtk_tree_model_get(model,iter,
		CACHE_COLUMN_NAME,&name,
		CACHE_COLUMN_INSTALLED,&iver,
		-1);
	if (*iver)
	{
		gchar *iname;
		gchar *pkgname;
		gyum_split_filename(name,&pkgname,NULL,NULL,NULL);
		iname=g_strdup_printf("%s-%s.rpm",
			pkgname,
			iver);
		if (strcmp(iname,name)) status=2;
		else status=1;
		g_free(iname);
		g_free(pkgname);
	}
	switch (status)
	{
		case 0:
			fg=&not_installed_fg;
			break;
		case 1:
			fg=&installed_fg;
			break;
		case 2:
			fg=&update_fg;
			break;
	}
	g_object_set(renderer,"foreground-set",TRUE,"foreground-gdk",fg,NULL);
	if (fontname)
		g_object_set(renderer,"font",fontname,NULL);
	g_free(name);
}

void gyum_cache_set_selection(GtkTreeViewColumn *column,GtkCellRendererToggle *renderer,
		GtkTreeModel *model,GtkTreeIter *iter,gpointer data)
{
	gboolean select;
	gtk_tree_model_get(model,iter,0,&select,-1);
	gtk_cell_renderer_toggle_set_active(renderer,select);
}

gboolean gyum_cache_pkglist_event(GtkTreeView *treeview,GdkEvent *event,GtkTreeModel *model)
{
	if (((event->type==GDK_BUTTON_RELEASE) && (event->button.button==1)) ||
		((event->type==GDK_KEY_PRESS) && (event->key.keyval==GDK_space)))
	{
		GtkTreePath *path;
		GtkTreeViewColumn *column;
		gtk_tree_view_get_cursor(treeview,&path,&column);
		if (path && 
			((event->type==GDK_KEY_PRESS) || 
			(column && !strcmp(gtk_tree_view_column_get_title(column),_("Selection")))))
		{
			GtkTreeIter iter;
			gboolean select;
			GtkTreeModel *sort_model;
			GtkTreePath *child_path;
			sort_model=gtk_tree_view_get_model(treeview);
			child_path=gtk_tree_model_sort_convert_path_to_child_path(
							GTK_TREE_MODEL_SORT(sort_model),path);
			gtk_tree_model_get_iter(model,&iter,child_path);
			gtk_tree_model_get(model,&iter,0,&select,-1);
			gtk_list_store_set(GTK_LIST_STORE(model),&iter,0,!select,-1);
			if (event->type==GDK_KEY_PRESS)
			{
				gtk_tree_path_next(path);
				gtk_tree_view_set_cursor(treeview,
					path,
					NULL,
					FALSE);
			}
			gtk_tree_path_free(path);
			gtk_tree_path_free(child_path);
			return TRUE;
		}
		gtk_tree_path_free(path);
	}
	return FALSE;
}


void gyum_cache_init(GtkWidget *window)
{
	gchar *pkglist_column_title[]={_("Selection"),_("Name"),_("Installed"),
			_("Size"),_("Repository")};
	gint i;
	GtkTreeModel *sort_model;
	yumrepo_get_cache_root();
	cache_pkglist=lookup_widget(window,"cache_pkglist");
	cache_pkglist_model=
		GTK_TREE_MODEL(gtk_list_store_new(CACHE_COLUMN_NUM,
						G_TYPE_BOOLEAN,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING));
	sort_model=gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(cache_pkglist_model));
	gtk_tree_view_set_model(GTK_TREE_VIEW(cache_pkglist),sort_model);
	g_signal_connect(cache_pkglist,"button-release-event",
			G_CALLBACK(gyum_cache_pkglist_event),cache_pkglist_model);
	g_signal_connect(cache_pkglist,"key-press-event",
			G_CALLBACK(gyum_cache_pkglist_event),cache_pkglist_model);
	for (i=0;i<CACHE_COLUMN_NUM;i++)
	{
		GtkCellRenderer *renderer;
		GtkTreeViewColumn *column;
		if (i==CACHE_COLUMN_SELECT)
		{
			renderer=gtk_cell_renderer_toggle_new();
			gtk_cell_renderer_toggle_set_radio(GTK_CELL_RENDERER_TOGGLE(renderer),FALSE);
			column=gtk_tree_view_column_new_with_attributes(
				pkglist_column_title[i],
				renderer,
				NULL);
			gtk_tree_view_column_set_cell_data_func(column,renderer,(GtkTreeCellDataFunc)gyum_cache_set_selection,NULL,NULL);
		}
		else
		{
			renderer=gtk_cell_renderer_text_new();
			column=gtk_tree_view_column_new_with_attributes(
				pkglist_column_title[i],
				renderer,
				"text",i,NULL);
			gtk_tree_view_column_set_cell_data_func(column,renderer,
				(GtkTreeCellDataFunc)gyum_cache_row_set,NULL,NULL);
		}
		gtk_tree_view_column_set_resizable(column,TRUE);
		gtk_tree_view_append_column(GTK_TREE_VIEW(cache_pkglist),column);
	}
	gtk_tree_view_set_search_column(GTK_TREE_VIEW(cache_pkglist),CACHE_COLUMN_NAME);
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sort_model),CACHE_COLUMN_NAME,GTK_SORT_ASCENDING);
}

static glong cache_bytes;
static glong cache_pkg_num;

void gyum_cache_reinit()
{
	cache_bytes=0;
	cache_pkg_num=0;
	gtk_list_store_clear(GTK_LIST_STORE(cache_pkglist_model));
}

void gyum_cache_fill(const gchar *dirname)
{
	gchar *str;
	gchar *size_str;
	DIR *dir;
	if (!cache_pkglist) return;
	if (!strcmp(dirname,yumrepo_get_cache_root()))
	{
		gyum_config_get_color("gui_not_installed_color",&not_installed_fg);
		gyum_config_get_color("gui_installed_color",&installed_fg);
		gyum_config_get_color("gui_update_color",&update_fg);
		if (fontname) g_free(fontname);
		fontname=gyum_config_get_string("list_font");
	}
	dir=opendir(dirname);
	if (dir)
	{
		struct dirent *de;
		while ((de=readdir(dir))!=NULL)
		{
			if (de->d_type==DT_DIR)
			{
				if (strcmp(de->d_name,".") && strcmp(de->d_name,".."))
				{
					gchar *fullname=g_strdup_printf("%s/%s",dirname,de->d_name);
					gyum_cache_fill(fullname);
					g_free(fullname);
				}
			}
			else
			if (de->d_type==DT_REG)
			{
				gchar *ptr=strstr(de->d_name,".rpm");
				if (ptr && !ptr[4])
				{
					GtkTreeIter iter;
					gchar *repo;
					gchar *ptr;
					struct stat st;
					gchar *fullname;
					gchar *pkgname;
					gchar *iver=NULL;
					const YUMPackage *pkg;
					fullname=g_strdup_printf("%s/%s",dirname,de->d_name);
					stat(fullname,&st);
					g_free(fullname);
					repo=g_strdup(dirname);
					ptr=strrchr(repo,'/');
					if (ptr)
					{
						*ptr=0;
						ptr=strrchr(repo,'/');
					}
					if (ptr) ptr++;
					cache_bytes+=st.st_size;
					if (st.st_size<1000)
						size_str=g_strdup_printf("%li Byte%s",st.st_size,st.st_size>1 ? "s":"");
					else
					if (st.st_size<1000000)
						size_str=g_strdup_printf("%li kB",st.st_size/1000);
					else
						size_str=g_strdup_printf("%li MB",st.st_size/1000000);
					gyum_split_filename(de->d_name,&pkgname,NULL,NULL,NULL);
					pkg=yumutils_get_package(pkgname);
					if (pkg && pkg->version[YUM_PACKAGE_STATUS_INSTALLED])
					{
						iver=g_strdup_printf("%s-%s.%s",
								pkg->version[YUM_PACKAGE_STATUS_INSTALLED],
								pkg->release[YUM_PACKAGE_STATUS_INSTALLED],
								pkg->arch);
					}
					g_free(pkgname);
					gtk_list_store_append(GTK_LIST_STORE(cache_pkglist_model),&iter);
					gtk_list_store_set(GTK_LIST_STORE(cache_pkglist_model),&iter,
						CACHE_COLUMN_SELECT,FALSE,
						CACHE_COLUMN_NAME,de->d_name,
						CACHE_COLUMN_INSTALLED,iver ? iver:"",
						CACHE_COLUMN_SIZE,size_str,
						CACHE_COLUMN_REPOSITORY,ptr ? ptr:"<unknown>",
						-1);
					cache_pkg_num++;
					g_free(size_str);
					g_free(repo);
				}
			}
		}
		closedir(dir);
	}
	if (cache_bytes<1000)
		size_str=g_strdup_printf("%li Byte%s",cache_bytes,cache_bytes>1 ? "s":"");
	else
		if (cache_bytes<1000000)
			size_str=g_strdup_printf("%li kB",cache_bytes/1000);
	else
		size_str=g_strdup_printf("%li MB",cache_bytes/1000000);
	str=g_strdup_printf(_("Allocated disk size by %li package(s): %s"),
			cache_pkg_num,size_str);
	gtk_label_set_text(GTK_LABEL(lookup_widget(cache_pkglist,"cache_status")),str);
	g_free(size_str);
	g_free(str);
}

typedef enum {
	CACHE_ERROR_NONE=0,
	CACHE_ERROR_SRCFILE_READ=1,
	CACHE_ERROR_DESTFILE_WRITE=2,
	CACHE_ERROR_SRCFILE_OPEN=3,
	CACHE_ERROR_DESTFILE_OPEN=4,
	CACHE_ERROR_SOURCE_REMOVE=5,
	CACHE_ERROR_DESTFILE_EXISTS=6,
	CACHE_ERROR_REMOVE_FILE=7,
	CACHE_ERROR_MEMORY_ALLOCATION=8
} CacheError;

typedef struct {
	GtkWidget *gyum_operation_message;
	GtkWidget *gyum_operation_progress;
	const gchar *destdir;
	gchar *src;
	gchar *dest;
	CacheError error;
} GYUMCacheOperation;

gboolean gyum_cache_move_one(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,GYUMCacheOperation *operation)
{
	gboolean select;
	gchar *name;
	gchar *repository;
	gboolean ret=FALSE;
	gtk_tree_model_get(model,iter,
		CACHE_COLUMN_SELECT,&select,
		CACHE_COLUMN_NAME,&name,
		CACHE_COLUMN_REPOSITORY,&repository,
		-1);
	if (select)
	{
		if (operation->destdir)
		{
			gchar *buffer;
			gchar *dest=NULL;
			gchar *src=NULL;
			src=g_strdup_printf("%s/%s/packages/%s",yumrepo_get_cache_root(),repository,name);
			dest=g_strdup_printf("%s/%s",operation->destdir,name);
			if (!access(dest,F_OK))
				operation->error=CACHE_ERROR_DESTFILE_EXISTS;
			if (access(src,W_OK))
				operation->error=CACHE_ERROR_SOURCE_REMOVE;
			else
			{
				buffer=g_new(gchar,65536);
				if (buffer)
				{
					FILE *srcfile=NULL;
					if (src)
					{
						srcfile=fopen(src,"rb");
						if (srcfile)
						{
							FILE *destfile=NULL;
							if (dest)
							{
								destfile=fopen(dest,"wb");
								if (destfile)
								{
									gulong length;
									gulong wb=0;
									fseek(srcfile,0,SEEK_END);
									length=ftell(srcfile);
									rewind(srcfile);
									gtk_label_set_text(GTK_LABEL(operation->gyum_operation_message),name);
									gtk_progress_set_value(GTK_PROGRESS(operation->gyum_operation_progress),0.0);
									gdk_window_process_all_updates();
									while (!feof(srcfile) &&
										   !ferror(srcfile) &&
										   !ferror(destfile))
									{
										gulong rb;
										rb=fread(buffer,1,65536,srcfile);
										fwrite(buffer,1,rb,destfile);
										wb+=rb;
										gtk_progress_set_value(GTK_PROGRESS(operation->gyum_operation_progress),wb*100/length);
										while (gtk_events_pending()) gtk_main_iteration();
									}
									if (ferror(srcfile)) operation->error=CACHE_ERROR_SRCFILE_READ;
									if (ferror(destfile)) operation->error=CACHE_ERROR_DESTFILE_WRITE;
									fclose(destfile);
								}
								else
									 operation->error=CACHE_ERROR_DESTFILE_OPEN;
							}
							else
								operation->error=CACHE_ERROR_MEMORY_ALLOCATION;
							fclose(srcfile);
							if (operation->error==CACHE_ERROR_NONE)
							{
								if (remove(src))
									operation->error=CACHE_ERROR_SOURCE_REMOVE;
							}
						}
						else
							operation->error=CACHE_ERROR_SRCFILE_OPEN;
					}
					else
						operation->error=CACHE_ERROR_MEMORY_ALLOCATION;
					g_free(buffer);
				}
				else
					operation->error=CACHE_ERROR_MEMORY_ALLOCATION;
			}
			if (operation->error!=CACHE_ERROR_NONE)
			{
				operation->src=src;
				operation->dest=dest;
				return TRUE;
			}
			else
			{
				g_free(src);
				g_free(dest);
			}
		}
		else
		{
			gchar *fn;
			fn=g_strdup_printf("%s/%s/packages/%s",yumrepo_get_cache_root(),repository,name);
			gtk_label_set_text(GTK_LABEL(operation->gyum_operation_message),name);
			gtk_progress_set_value(GTK_PROGRESS(operation->gyum_operation_progress),0.0);
			while (gtk_events_pending()) gtk_main_iteration();
			ret=remove(fn);
			operation->error=CACHE_ERROR_REMOVE_FILE;
			gtk_progress_set_value(GTK_PROGRESS(operation->gyum_operation_progress),100.0);
			gdk_window_process_all_updates();
		}
	}
	g_free(name);
	g_free(repository);
	return ret;
}

void gyum_cache_pkgs_move(const gchar *destdir)
{
	GYUMCacheOperation operation;
	gchar *title;
	GtkWidget *gyum_operation;
	gyum_operation=create_gyum_operation();
	operation.gyum_operation_message=lookup_widget(gyum_operation,"gyum_operation_message");
	operation.gyum_operation_progress=lookup_widget(gyum_operation,"gyum_operation_progress");
	operation.destdir=destdir;
	operation.error=CACHE_ERROR_NONE;
	title=g_strdup_printf(_("Move selected packages from cache to %s"),destdir);
	gtk_window_set_title(GTK_WINDOW(gyum_operation),title);
	gtk_widget_show(gyum_operation);
	g_free(title);
	gtk_tree_model_foreach(cache_pkglist_model,
				(GtkTreeModelForeachFunc)gyum_cache_move_one,&operation);
	if (operation.error!=CACHE_ERROR_NONE)
	{
		static const gchar *errorstr[]={
			NULL,
			"Source file reading error",
			"Destination file writing error",
			"Source file opening error",
			"Destination file opening error",
			"Source file deleting error",
			"Source file exists",
			"File removing error",
			"Memory allocation error"};
		GtkWidget *error_widget;
		error_widget=gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,
				GTK_BUTTONS_OK,
				"error: %s\n%s\n\nsource file:\n%s\n\ndestination file:\n%s",
				errorstr[operation.error],
				operation.error<CACHE_ERROR_MEMORY_ALLOCATION ? strerror(errno):"",
				operation.src,operation.dest);
		gtk_dialog_run(GTK_DIALOG(error_widget));
		gtk_widget_destroy(error_widget);
		g_free(operation.src);
		g_free(operation.dest);
	}
	gtk_widget_destroy(gyum_operation);
	gyum_cache_reinit();
	gyum_cache_fill(yumrepo_get_cache_root());
}

void gyum_cache_pkgs_remove()
{
	GYUMCacheOperation operation;
	GtkWidget *gyum_operation;
	gyum_operation=create_gyum_operation();
	operation.gyum_operation_message=lookup_widget(gyum_operation,"gyum_operation_message");
	operation.gyum_operation_progress=lookup_widget(gyum_operation,"gyum_operation_progress");
	operation.destdir=NULL;
	gtk_window_set_title(GTK_WINDOW(gyum_operation),_("Remove selected packages from cache"));
	gtk_widget_show(gyum_operation);
	gtk_tree_model_foreach(cache_pkglist_model,
				(GtkTreeModelForeachFunc)gyum_cache_move_one,&operation);
	gtk_widget_destroy(gyum_operation);
	gyum_cache_reinit();
	gyum_cache_fill(yumrepo_get_cache_root());
}

gboolean gyum_cache_selected(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,GList **list)
{
	gboolean select;
	gchar *name;
	gchar *repository;
	gtk_tree_model_get(model,iter,
		CACHE_COLUMN_SELECT,&select,
		CACHE_COLUMN_NAME,&name,
		CACHE_COLUMN_REPOSITORY,&repository,
		-1);
	if (select)
	{
		gchar *fullpath;
		fullpath=g_strdup_printf("%s/%s/packages/%s",yumrepo_get_cache_root(),repository,name);
		*list=g_list_append(*list,fullpath);
	}
	g_free(name);
	g_free(repository);
	return FALSE;
}

GList *gyum_cache_pkgs_selected()
{
	GList *list=NULL;
	gtk_tree_model_foreach(cache_pkglist_model,
				(GtkTreeModelForeachFunc)gyum_cache_selected,&list);
	return list;
}

void gyum_cache_clean(GyumCacheClean clean)
{
	const gchar *cmd;
	switch (clean)
	{
		case GYUM_CACHE_CLEAN_HEADERS:
			cmd="yum clean headers";
			break;
		case GYUM_CACHE_CLEAN_PACKAGES:
			cmd="yum clean packages";
			break;
		case GYUM_CACHE_CLEAN_ALL:
			cmd="yum clean all";
			break;
		default:
			return;
	}
	gyum_command_execute(_("Cleaning cache"),cmd,FALSE,NULL,NULL);
}


void gyum_cache_unload(GtkWidget *window)
{

}

void gyum_cache_cleaner_set_sensitive(gboolean sensitive)
{
	if (cache_pkglist)
	{
		GtkWidget *clean_headers;
		GtkWidget *clean_packages;
		clean_headers=lookup_widget(cache_pkglist,"bt_cache_clean_headers");
		clean_packages=lookup_widget(cache_pkglist,"bt_cache_clean_packages");
		gtk_widget_set_sensitive(clean_headers,sensitive);
		gtk_widget_set_sensitive(clean_packages,sensitive);
	}
}

gchar *gyum_cache_get_current_package()
{
	GtkTreePath *path;
	gchar *pkg=NULL;
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(cache_pkglist),&path,NULL);
	if (path)
	{
		GtkTreeIter iter;
		GtkTreeIter child_iter;
		GtkTreeModel *sort_model;
		gchar *name;
		gchar *repo;
		sort_model=gtk_tree_view_get_model(GTK_TREE_VIEW(cache_pkglist));
		gtk_tree_model_get_iter(sort_model,&iter,path);
		gtk_tree_path_free(path);
		gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT(sort_model),
			&child_iter,&iter);
		gtk_tree_model_get(cache_pkglist_model,&child_iter,
			CACHE_COLUMN_NAME,&name,
			CACHE_COLUMN_REPOSITORY,&repo,-1);
		pkg=g_strdup_printf("%s/%s/packages/%s",yumrepo_get_cache_root(),repo,name);
		g_free(name);
		g_free(repo);
	}
	return pkg;
}
