#include <string.h>
#include <stdlib.h>
#include "gyum_config.h"

static GConfClient *gyum_conf=NULL;
static GHashTable *default_values=NULL;

void gyum_config_connect()
{
	if (!gyum_conf)
	{
		typedef struct {
			const gchar *var;
			const gchar *val;
		} GYUMConfigValue;
		static  GYUMConfigValue gyum_config_default_values[]=
			{{"loading_startup","0"},
			 {"default_filter","updates"},
			 {"distributor_packages","1"},
			 {"external_packages","1"},
			 {"gui_sensitivity","100"},
			 {"show_splash_window","1"},
			 {"terminal_background","#ffffff"},
			 {"terminal_foreground","#000000"},
			 {"rpminfo_library_paths","/lib,/usr/lib,/usr/local/lib,/usr/X11R6/lib"},
			 {"rpminfo_binary_paths","/bin,/sbin,/usr/bin,/usr/sbin,/usr/libexec,/usr/local/bin,/usr/local/sbin,/usr/local/libexec,/usr/X11R6/bin"},
			 {"rpm_directory","/usr/src/redhat/RPMS"},
			 {"gui_not_installed_color","black"},
			 {"gui_installed_color","gray"},
			 {"gui_update_color","blue"},
			 {NULL,NULL}};
		gint i;
		gyum_conf=gconf_client_get_default();
		gconf_client_add_dir(gyum_conf,"/apps/gnome-yum",GCONF_CLIENT_PRELOAD_RECURSIVE,NULL);
		default_values=g_hash_table_new_full(g_str_hash,g_str_equal,g_free,g_free);
		for (i=0;gyum_config_default_values[i].var;i++)
		{
			g_hash_table_insert(default_values,
				g_strdup(gyum_config_default_values[i].var),
				g_strdup(gyum_config_default_values[i].val));
		}
	}
}

void gyum_config_set_string(const gchar *key,const gchar *val)
{
	gchar *path;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	gconf_client_set_string(gyum_conf,path,val,NULL);
	g_free(path);
}

GConfValue *gyum_config_set_string_value(const gchar *val)
{
	GConfValue *gval;
	gval=gconf_value_new(GCONF_VALUE_STRING);
	gconf_value_set_string(gval,val);
	return gval;
}

gchar *gyum_config_get_string(const gchar *key)
{
	gchar *path;
	gchar *ret;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	ret=gconf_client_get_string(gyum_conf,path,NULL);
	g_free(path);
	if (!ret)
	{
		ret=g_hash_table_lookup(default_values,key);
		if (ret) ret=g_strdup(ret);
	}
	return ret;
}

void gyum_config_set_boolean(const gchar *key,gboolean val)
{
	gchar *path;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	gconf_client_set_bool(gyum_conf,path,val,NULL);
	g_free(path);
}

GConfValue *gyum_config_set_boolean_value(gboolean val)
{
	GConfValue *gval;
	gval=gconf_value_new(GCONF_VALUE_BOOL);
	gconf_value_set_bool(gval,val);
	return gval;
}

gboolean gyum_config_get_boolean(const gchar *key)
{
	gchar *path;
	const GConfValue *val;
	gchar *val_str;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	val=gconf_client_get(gyum_conf,path,NULL);
	g_free(path);
	if (val)
		return gconf_value_get_bool(val);
	val_str=g_hash_table_lookup(default_values,key);
	return val_str && !strcmp(val_str,"1") ? TRUE:FALSE;
}

void gyum_config_set_int(const gchar *key,gint val)
{
	gchar *path;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	gconf_client_set_int(gyum_conf,path,val,NULL);
	g_free(path);
}

GConfValue *gyum_config_set_int_value(gint val)
{
	GConfValue *gval;
	gval=gconf_value_new(GCONF_VALUE_INT);
	gconf_value_set_int(gval,val);
	return gval;
}

gint gyum_config_get_int(const gchar *key)
{
	gchar *path;
	const GConfValue *val;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	val=gconf_client_get(gyum_conf,path,NULL);
	if (!val)
	{
		gint val=0;
		gchar *val_str;
		val_str=g_hash_table_lookup(default_values,key);
		if (val_str)
			val=atoi(val_str);
		return val;
	}
	g_free(path);
	return gconf_value_get_int(val);
}

void gyum_config_set_color(const gchar *key,const GdkColor *color)
{
	gchar *path;
	gchar *val;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	val=g_strdup_printf("#%02x%02x%02x",color->red>>8,color->green>>8,color->blue>>8);
	gconf_client_set_string(gyum_conf,path,val,NULL);
	g_free(path);
	g_free(val);
}

GConfValue *gyum_config_set_color_value(const GdkColor *color)
{
	GConfValue *gval;
	gchar *val;
	gval=gconf_value_new(GCONF_VALUE_STRING);
	val=g_strdup_printf("#%02x%02x%02x",color->red>>8,color->green>>8,color->blue>>8);
	gconf_value_set_string(gval,val);
	return gval;
}

GdkColor *gyum_config_get_color(const gchar *key,GdkColor *color)
{
	gchar *path;
	gchar *ret;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	ret=gconf_client_get_string(gyum_conf,path,NULL);
	if (!ret)
	{
		ret=g_hash_table_lookup(default_values,key);
		if (ret) ret=g_strdup(ret);
	}
	if (ret) gdk_color_parse(ret,color);
	else color=NULL;
	g_free(path);
	g_free(ret);
	return color;
}

void gyum_config_set_list(const gchar *key,GSList *list)
{
	gchar *path;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	gconf_client_set_list(gyum_conf,path,GCONF_VALUE_STRING,list,NULL);
	g_free(path);
}

GConfValue *gyum_config_set_list_value(GSList *list)
{
	GConfValue *gval;
	gval=gconf_value_new(GCONF_VALUE_LIST);
	gconf_value_set_list(gval,list);
	return gval;
}

GSList *gyum_config_get_list(const gchar *key)
{
	gchar *path;
	GSList *ret;
	gyum_config_connect();
	path=g_strdup_printf("/apps/gnome-yum/%s",key);
	ret=gconf_client_get_list(gyum_conf,path,GCONF_VALUE_STRING,NULL);
	if (!ret)
	{
		const gchar *str;
		str=g_hash_table_lookup(default_values,key);
		if (str)
		{
			gchar **strv;
			strv=g_strsplit(str,",",0);
			if (strv)
			{
				int i;
				for (i=0;strv[i];i++)
					ret=g_slist_append(ret,g_strdup(strv[i]));
				g_strfreev(strv);
			}
		}
	}
	g_free(path);
	return ret;
}
