#include <string.h>
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include "support.h"
#include "yumrepo.h"
#include "gyum_appbar.h"
#include "gyum_utils.h"
#include "gyum_config.h"
#include "gyum_cache.h"
#include "gyum_pkglist.h"

#define CMD_CHECK_UPDATES	("/bin/sh " PACKAGE_DATA_DIR "/gnome-yum/gyum-query.sh -c")

static gboolean user_root=FALSE;
static GtkTreeView *treeview=NULL;
static GtkTreeModel *treeview_list=NULL;
static gboolean pkglist_invert=FALSE;
static PkgListFilter pkglist_filter=
	PKG_LIST_FILTER_ALL|PKG_LIST_FILTER_DISTRIBUTOR|PKG_LIST_FILTER_OTHER;
static PkgListSearch pkglist_search;
static GPatternSpec *pkglist_pspec=NULL;
static GtkWidget *op_buttons[APPBAR_STATUS_ITEM_NUM];

enum {
	COLUMN_NAME,
	COLUMN_INSTALLED,
	COLUMN_NEW,
	COLUMN_REPOSITORY,
	COLUMN_OPERATION,
	COLUMN_N
};

static GdkColor not_installed_fg;
static GdkColor installed_fg;
static GdkColor update_fg;
static gchar *fontname=NULL;

void gyum_pkglist_row_set(GtkTreeViewColumn *column,GtkCellRenderer *renderer,GtkTreeModel *model,GtkTreeIter *iter,gpointer data)
{
	gchar *iver;
	gchar *nver;
	GdkColor *fg;
	gtk_tree_model_get(model,iter,COLUMN_INSTALLED,&iver,COLUMN_NEW,&nver,-1);
	if (*iver)
	{
		if (*nver)
			fg=&update_fg;
		else
			fg=&installed_fg;
	}
	else
		fg=&not_installed_fg;
	g_object_set(renderer,"foreground-set",TRUE,"foreground-gdk",fg,NULL);
	if (fontname)
		g_object_set(renderer,"font",fontname,NULL);
	g_free(iver);
	g_free(nver);
}

void gyum_pkglist_init(GtkWidget *tv)
{
	gchar *column_title[]=
		{_("Name"),_("Installed"),_("New"),_("Repository"),_("Operation")};
	int i;
	GtkListStore *store;
	user_root=!strcmp(getenv("USER"),"root");
	treeview=GTK_TREE_VIEW(tv);
	store=gtk_list_store_new(5,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
	treeview_list=gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
	gtk_tree_view_set_model(treeview,treeview_list);
	for (i=0;i<COLUMN_N;i++)
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *renderer;
		renderer=gtk_cell_renderer_text_new();
		column=gtk_tree_view_column_new_with_attributes(column_title[i],renderer,"text",i,NULL);
		gtk_tree_view_column_set_resizable(column,TRUE);
		gtk_tree_view_append_column(treeview,column);
		gtk_tree_view_column_set_cell_data_func(column,renderer,
			(GtkTreeCellDataFunc)gyum_pkglist_row_set,NULL,NULL);
	}
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(treeview_list),COLUMN_NAME,GTK_SORT_ASCENDING);
	op_buttons[APPBAR_STATUS_ITEM_SELECT_INSTALL]=lookup_widget(GTK_WIDGET(treeview),"bt_op_install");
	op_buttons[APPBAR_STATUS_ITEM_SELECT_UPDATE]=lookup_widget(GTK_WIDGET(treeview),"bt_op_update");
	op_buttons[APPBAR_STATUS_ITEM_SELECT_REMOVE]=lookup_widget(GTK_WIDGET(treeview),"bt_op_remove");
}

void gyum_pkglist_free()
{
	yumutils_free();
}

void gyum_pkglist_filter(PkgListFilter status,PkgListFilter provider,gboolean invert,gboolean set_default)
{
	PkgListFilter filter=status|provider;
	if ((filter!=pkglist_filter) || (pkglist_invert!=invert))
	{
		pkglist_filter=filter;
		pkglist_invert=invert;
		if (!set_default) gyum_pkglist_fill();
	}
}

void gyum_pkglist_search(PkgListSearch search,const gchar *str)
{
	pkglist_search=search;
	if (!strcmp(str,"*") && (search<PKG_LIST_SEARCH_SELECTIONS_FOR_INSTALL))
	{
		if (pkglist_pspec)
		{
			g_pattern_spec_free(pkglist_pspec);
			pkglist_pspec=NULL;
		}
	}
	else
		pkglist_pspec=g_pattern_spec_new(str);
	gyum_pkglist_fill();
}

gboolean gyum_pkglist_pkg_ok(YUMPackage *yum)
{
	gboolean ok=FALSE;
	if ((yum->status&PKG_LIST_FILTER_STATUS_MASK)==(pkglist_filter&PKG_LIST_FILTER_STATUS_MASK)) ok=TRUE;
	if (((yum->status&PKG_LIST_FILTER_STATUS_MASK)>=(pkglist_filter&PKG_LIST_FILTER_STATUS_MASK)) &&
		((pkglist_filter&PKG_LIST_FILTER_STATUS_MASK)!=PKG_LIST_FILTER_NOT_INSTALLED)) ok=TRUE;
	if (ok)
	{
		if (yum->status&(pkglist_filter&PKG_LIST_FILTER_PROVIDER_MASK))
		{
			if (pkglist_pspec)
			{	
				GList *search_list=NULL;
				GList *p;
				ok=FALSE;
				switch (pkglist_search)
				{
					case PKG_LIST_SEARCH_ALL:
						search_list=g_list_append(search_list,yum->name);
						search_list=g_list_append(search_list,yum->repository);
						search_list=g_list_append(search_list,yum->summary);
						search_list=g_list_append(search_list,yum->description);
						break;
					case PKG_LIST_SEARCH_PACKAGE_NAME:
						search_list=g_list_append(search_list,yum->name);
						break;
					case PKG_LIST_SEARCH_REPOSITORY:
						search_list=g_list_append(search_list,yum->repository);
						break;
					case PKG_LIST_SEARCH_SUMMARY_N_DESCRIPTION:
						search_list=g_list_append(search_list,yum->summary);
						search_list=g_list_append(search_list,yum->description);
						break;
					case PKG_LIST_SEARCH_SELECTIONS_FOR_INSTALL:
						if (!strcmp(yum->operation,"Install"))
							search_list=g_list_append(search_list,yum->name);
						break;
					case PKG_LIST_SEARCH_SELECTIONS_FOR_UPDATE:
						if (!strcmp(yum->operation,"Update"))
							search_list=g_list_append(search_list,yum->name);
						break;
					case PKG_LIST_SEARCH_SELECTIONS_FOR_REMOVE:
						if (!strcmp(yum->operation,"Remove"))
							search_list=g_list_append(search_list,yum->name);
						break;
				}
				p=search_list;
				while (p)
				{
					if (g_pattern_match_string(pkglist_pspec,(gchar *)p->data)!=pkglist_invert)
					{
						ok=TRUE;
						break;
					}
					p=p->next;
				}
				g_list_free(search_list);
			}
		}
		else
			ok=FALSE;
	}
	return ok;
}

static gint gyum_package_counter=0;
static gint gyum_gui_sensitivity=100;

void gyum_yum_package(const gchar *name,YUMPackage *yum,GtkListStore *list)
{
	static gint counter=0;
	if (!yum) abort();
	if (yum->version[YUM_PACKAGE_STATUS_INSTALLED])
		gyum_appbar_status_increment(APPBAR_STATUS_ITEM_INSTALLED,1);
	gyum_appbar_status_increment(APPBAR_STATUS_ITEM_AVAILABLE,1);
	if (gyum_pkglist_pkg_ok(yum))
	{
		GtkTreeIter iter;
		gchar *iver=NULL;
		gchar *nver=NULL;
		if (yum->version[YUM_PACKAGE_STATUS_INSTALLED])
		{
			iver=g_strdup_printf("%s%s%s",yum->version[YUM_PACKAGE_STATUS_INSTALLED],
					yum->release[YUM_PACKAGE_STATUS_INSTALLED] ? "-":"",
					yum->release[YUM_PACKAGE_STATUS_INSTALLED] ? yum->release[YUM_PACKAGE_STATUS_INSTALLED]:"");
		}
		if (yum->version[YUM_PACKAGE_STATUS_AVAILABLE])
		{
			nver=g_strdup_printf("%s%s%s",yum->version[YUM_PACKAGE_STATUS_AVAILABLE],
					yum->release[YUM_PACKAGE_STATUS_AVAILABLE] ? "-":"",
					yum->release[YUM_PACKAGE_STATUS_AVAILABLE] ? yum->release[YUM_PACKAGE_STATUS_AVAILABLE]:"");
		}
		gtk_list_store_append(list,&iter);
		gtk_list_store_set(list,&iter,
			COLUMN_NAME,name,
			COLUMN_INSTALLED,iver ? iver:"",
			COLUMN_NEW,nver ? nver:"",
			COLUMN_REPOSITORY,yum->repository,
			COLUMN_OPERATION,yum->operation,
			-1);
		if (!strcmp(yum->operation,"Install"))
			gyum_appbar_status_increment(APPBAR_STATUS_ITEM_SELECT_INSTALL,1);
		else
		if (!strcmp(yum->operation,"Update"))
			gyum_appbar_status_increment(APPBAR_STATUS_ITEM_SELECT_UPDATE,1);
		else
		if (!strcmp(yum->operation,"Remove"))
			gyum_appbar_status_increment(APPBAR_STATUS_ITEM_SELECT_REMOVE,1);
		gyum_appbar_status_increment(APPBAR_STATUS_ITEM_DISPLAYED,1);
		g_free(iver);
		g_free(nver);
	}
	counter++;
	gyum_package_counter++;
	if (counter>=gyum_gui_sensitivity)
	{
		gyum_appbar_progress_set_fraction((gfloat)gyum_package_counter/yumutils_size());
		while (gtk_events_pending()) gtk_main_iteration();
		counter=0;
	}
}

static gboolean gyum_pkglist_load_op=FALSE;

void gyum_check_updates_done(pid_t pid,gint status,gpointer data)
{
	yumutils_package_list(gyum_pkglist_fill);
}

void gyum_pkglist_check_updates()
{
	if (!gyum_pkglist_load_op)
	{
		gyum_pkglist_load_op=TRUE;
		gyum_controllers_set_sensitive(FALSE);
		gyum_appbar_progress_begin();
		gtk_notebook_set_current_page(
			GTK_NOTEBOOK(lookup_widget(GTK_WIDGET(treeview),"gyum_notebook")),1);
		gyum_command_execute(_("Checking updates"),CMD_CHECK_UPDATES,TRUE,gyum_check_updates_done,NULL);
	}
}

void gyum_pkglist_load_done(gint status)
{
	gyum_pkglist_fill();
	if (status)
	{
		gyum_pkglist_load_op=FALSE;
		gyum_controllers_set_sensitive(TRUE);
		gyum_cache_cleaner_set_sensitive(TRUE);
		gyum_appbar_display_status();
	}
}

void gyum_pkglist_load()
{
	if (!gyum_pkglist_load_op)
	{
		gyum_pkglist_load_op=TRUE;
		gyum_controllers_set_sensitive(FALSE);
		gyum_cache_cleaner_set_sensitive(FALSE);
		gyum_appbar_progress_begin();
		gtk_notebook_set_current_page(
			GTK_NOTEBOOK(lookup_widget(GTK_WIDGET(treeview),"gyum_notebook")),1);
		yumutils_package_list(gyum_pkglist_load_done);
	}
}

void gyum_pkglist_fill()
{
	if (!treeview) return;
	gyum_appbar_progress_end();
	gyum_controllers_set_sensitive(FALSE);
	gyum_cache_cleaner_set_sensitive(FALSE);
	gyum_appbar_reset_status();
	if (!yumutils_package_list_available())
	{
		gyum_pkglist_load();
	}
	else
	{
		static const gchar *entry_names[]={
			"pkg_name","pkg_summary","pkg_version","pkg_release","pkg_size",
			"available_version","available_release","available_repository","available_size",
			NULL};
		GtkListStore *store;
		GtkTextBuffer *textbuffer;
		int i;
		GtkTreeIter iter;
		GtkTreePath *path;
		gtk_notebook_set_current_page(
			GTK_NOTEBOOK(lookup_widget(GTK_WIDGET(treeview),"gyum_notebook")),0);
		store=GTK_LIST_STORE(gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(treeview_list)));
		gtk_list_store_clear(store);
		gyum_package_counter=0;
		for (i=0;entry_names[i];i++)
		{
			gtk_entry_set_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),entry_names[i])),"");
		}
		textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(treeview),"pkg_description")));
		gtk_text_buffer_set_text(textbuffer,"",-1);
		gyum_gui_sensitivity=gyum_config_get_int("gui_sensitivity");
		gyum_appbar_progress_set_fraction(0.0);
		gyum_config_get_color("gui_not_installed_color",&not_installed_fg);
		gyum_config_get_color("gui_installed_color",&installed_fg);
		gyum_config_get_color("gui_update_color",&update_fg);
		if (fontname) g_free(fontname);
		fontname=gyum_config_get_string("list_font");
		if (yumutils_file_exists(".last"))
		{
			GtkWidget *msg;
			gint ret;
			msg=gtk_message_dialog_new(NULL,
				GTK_DIALOG_MODAL,
				GTK_MESSAGE_WARNING,
				GTK_BUTTONS_YES_NO,
				_("Previous execute of the program is crashed or the last operation is unsuccessful.\nDo you want to load previous operation selections?"));
			ret=gtk_dialog_run(GTK_DIALOG(msg));
			gtk_widget_destroy(msg);
			switch (ret)
			{
				case GTK_RESPONSE_YES:
					yumutils_file_load(".last");
					break;
			}
			yumutils_file_remove(".last");
		}
		yumutils_foreach((YPFunc)gyum_yum_package,store);
		gyum_pkglist_load_op=FALSE;
		gyum_controllers_set_sensitive(TRUE);
		gyum_cache_cleaner_set_sensitive(TRUE);
		gyum_appbar_display_status();
		gtk_tree_view_get_cursor(treeview,&path,NULL);
		if (!path)
		{
			gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store),&iter);
			path=gtk_tree_model_get_path(GTK_TREE_MODEL(store),&iter);
			gtk_tree_view_set_cursor(treeview,path,NULL,FALSE);
		}
		else
			gtk_tree_view_scroll_to_cell(treeview,path,NULL,TRUE,0.5,0.0);
		gtk_tree_path_free(path);
		gyum_appbar_progress_set_fraction(1.0);
		gyum_cache_reinit();
		gyum_cache_fill(yumrepo_get_cache_root());
	}
}

static gboolean command_execute=FALSE;

void gyum_pkglist_command_begin()
{
	command_execute=TRUE;
}

void gyum_pkglist_command_end()
{
	command_execute=FALSE;
	gtk_widget_set_sensitive(
		lookup_widget(GTK_WIDGET(treeview),"fr_operation"),
		TRUE);
}

#define CHECK_STR(__str) (__str ? __str:"")

void gyum_pkglist_cursor_changed()
{
	const YUMPackage *pkg;
	GtkTreeIter iter;
	GtkTreePath *path;
	GtkTextBuffer *textbuffer;
	gchar *name;
	gchar *op;
	gtk_tree_view_get_cursor(treeview,&path,NULL);
	gtk_tree_model_get_iter(GTK_TREE_MODEL(treeview_list),&iter,path);
	gtk_tree_model_get(GTK_TREE_MODEL(treeview_list),&iter,
			COLUMN_NAME,&name,
			COLUMN_OPERATION,&op,
			-1);
	gtk_tree_path_free(path);
	pkg=yumutils_get_package(name);
	if (pkg)
	{
		gchar *rb_name;
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"pkg_name")),pkg->name);
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"pkg_summary")),pkg->summary ? pkg->summary:"<not available>");
		textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(treeview),"pkg_description")));
		gtk_text_buffer_set_text(textbuffer,pkg->description ? pkg->description:"<not available>",-1);
	
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"pkg_version")),CHECK_STR(pkg->version[YUM_PACKAGE_STATUS_INSTALLED]));
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"pkg_release")),CHECK_STR(pkg->release[YUM_PACKAGE_STATUS_INSTALLED]));
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"pkg_size")),CHECK_STR(pkg->size[YUM_PACKAGE_STATUS_INSTALLED]));
	
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"available_version")),CHECK_STR(pkg->version[YUM_PACKAGE_STATUS_AVAILABLE]));
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"available_release")),CHECK_STR(pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]));
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"available_size")),CHECK_STR(pkg->size[YUM_PACKAGE_STATUS_AVAILABLE]));
		if (pkg->version[YUM_PACKAGE_STATUS_AVAILABLE])
			gtk_entry_set_text(
				GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"available_repository")),CHECK_STR(pkg->repository));
		else
			gtk_entry_set_text(
				GTK_ENTRY(lookup_widget(GTK_WIDGET(treeview),"available_repository")),"");
	
		rb_name=g_strdup_printf("rb_op_%s",op);
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(treeview),rb_name)),TRUE);
		
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"tb_information"),
				pkg->version[YUM_PACKAGE_STATUS_INSTALLED]!=NULL);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"fr_operation"),
				!command_execute & user_root & TRUE);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"rb_op_None"),TRUE);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"rb_op_Install"),
			(pkg->status&PKG_LIST_FILTER_STATUS_MASK)==PKG_LIST_FILTER_NOT_INSTALLED);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"rb_op_Update"),
			(pkg->status&PKG_LIST_FILTER_STATUS_MASK)>=PKG_LIST_FILTER_UPDATES);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(treeview),"rb_op_Remove"),
			(pkg->status&PKG_LIST_FILTER_STATUS_MASK)!=PKG_LIST_FILTER_NOT_INSTALLED);
		g_free(rb_name);
	}
	
	g_free(op);
	g_free(name);
}

static const gchar *gyum_pkglist_operation[]={"None","Install","Update","Remove"};

void gyum_pkglist_op_button_set_sensitive(AppBarStatusItem status,gboolean on)
{
	if (op_buttons[status])
	{
		if (!gyum_appbar_get_status(status))
			gtk_widget_set_sensitive(op_buttons[status],FALSE);
		else
			gtk_widget_set_sensitive(op_buttons[status],on && TRUE);
	}
}

gboolean gyum_pkglist_set_operation_for_package(const gchar *name,PkgListOperation operation)
{
	YUMPackage *pkg;
	AppBarStatusItem appbar_status=APPBAR_STATUS_ITEM_UNKNOWN;
	gchar *old_operation;
	pkg=(YUMPackage *)yumutils_get_package(name);
	if (pkg)
	{
		old_operation=pkg->operation;
		switch (operation)
		{
			case PKG_LIST_OPERATION_NONE:
				pkg->operation=g_strdup(gyum_pkglist_operation[operation]);
				break;
			case PKG_LIST_OPERATION_INSTALL:
				if ((pkg->status&PKG_LIST_FILTER_STATUS_MASK)==PKG_LIST_FILTER_NOT_INSTALLED)
					pkg->operation=g_strdup(gyum_pkglist_operation[operation]);
				else
					return FALSE;
				break;
			case PKG_LIST_OPERATION_UPDATE:
				if ((pkg->status&PKG_LIST_FILTER_STATUS_MASK)>=PKG_LIST_FILTER_UPDATES)
					pkg->operation=g_strdup(gyum_pkglist_operation[operation]);
				else
					return FALSE;
				break;
			case PKG_LIST_OPERATION_REMOVE:
				if ((pkg->status&PKG_LIST_FILTER_STATUS_MASK)>=PKG_LIST_FILTER_INSTALLED)
					pkg->operation=g_strdup(gyum_pkglist_operation[operation]);
				else
					return FALSE;
				break;
		}
		if (!strcmp(old_operation,"Install"))
			appbar_status=APPBAR_STATUS_ITEM_SELECT_INSTALL;
		else
		if (!strcmp(old_operation,"Update"))
			appbar_status=APPBAR_STATUS_ITEM_SELECT_UPDATE;
		else
		if (!strcmp(old_operation,"Remove"))
			appbar_status=APPBAR_STATUS_ITEM_SELECT_REMOVE;
		if (appbar_status!=APPBAR_STATUS_ITEM_UNKNOWN)
		{
			gyum_appbar_status_increment(appbar_status,-1);
			gyum_pkglist_op_button_set_sensitive(appbar_status,TRUE);
			appbar_status=APPBAR_STATUS_ITEM_UNKNOWN;
		}
		switch (operation)
		{
			case PKG_LIST_OPERATION_NONE:
				break;
			case PKG_LIST_OPERATION_INSTALL:
				appbar_status=APPBAR_STATUS_ITEM_SELECT_INSTALL;
				break;
			case PKG_LIST_OPERATION_UPDATE:
				appbar_status=APPBAR_STATUS_ITEM_SELECT_UPDATE;
				break;
			case PKG_LIST_OPERATION_REMOVE:
				appbar_status=APPBAR_STATUS_ITEM_SELECT_REMOVE;
				break;
		}
		if (appbar_status!=APPBAR_STATUS_ITEM_UNKNOWN)
		{
			gyum_appbar_status_increment(appbar_status,1);
			gyum_pkglist_op_button_set_sensitive(appbar_status,TRUE);
		}
		gyum_appbar_display_status();
		g_free(old_operation);
		return TRUE;
	}
	else
	{
		g_warning(_("Information of the %s package not found."),name);
		return FALSE;
	}
}

void gyum_pkglist_set_operation(PkgListOperation operation)
{
	GtkTreePath *path;
	GtkTreeViewColumn *column;
	GtkTreeIter iter;
	GtkTreeIter child_iter;
	gchar *name;
	GtkListStore *store;
	store=GTK_LIST_STORE(gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(treeview_list)));
	gtk_tree_view_get_cursor(treeview,&path,&column);
	gtk_tree_model_get_iter(treeview_list,&iter,path);
	gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT(treeview_list),&child_iter,&iter);
	gtk_tree_model_get(
		GTK_TREE_MODEL(store),&child_iter,
		COLUMN_NAME,&name,
		-1);
	if (gyum_pkglist_set_operation_for_package(name,operation))
	{
		gtk_list_store_set(
			store,&child_iter,
			COLUMN_OPERATION,gyum_pkglist_operation[operation],
			-1);
	}
	g_free(name);
}

gboolean gyum_pkglist_set_operation_for_one(GtkTreeModel *model,GtkTreePath *path,
		GtkTreeIter *iter,PkgListOperation operation)
{
	gchar *name;
	gtk_tree_model_get(model,iter,COLUMN_NAME,&name,-1);
	if (gyum_pkglist_set_operation_for_package(name,operation))
	{
		gtk_list_store_set(
			GTK_LIST_STORE(model),iter,
			COLUMN_OPERATION,gyum_pkglist_operation[operation],
			-1);
	}
	g_free(name);
	return FALSE;
}

void gyum_pkglist_set_operation_for_all(PkgListOperation operation)
{
	gtk_tree_model_foreach(gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(treeview_list)),
			(GtkTreeModelForeachFunc)gyum_pkglist_set_operation_for_one,(gpointer)operation);
}

typedef struct {
	int count;
	const gchar *operation;
	GString *command;
	GList *repo;
} PkgListForeachData;

gboolean gyum_pkglist_foreach_func(GtkTreeModel *model,
	GtkTreePath *path,GtkTreeIter *iter,PkgListForeachData *data)
{
	gchar *name;
	gchar *operation;
	gchar *repo;
	gtk_tree_model_get(model,iter,
		COLUMN_NAME,&name,
		COLUMN_REPOSITORY,&repo,
		COLUMN_OPERATION,&operation,-1);
	if (!strcmp(operation,data->operation))
	{
		GList *i;
		g_string_append_printf(data->command," %s",name);
		i=g_list_find_custom(data->repo,repo,(GCompareFunc)strcmp);
		if (!i) data->repo=g_list_append(data->repo,g_strdup(repo));
		data->count++;
	}
	g_free(name);
	g_free(repo);
	g_free(operation);
	return FALSE;
}

void gyum_pkglist_check_repo(gchar *repo,GHashTable *repolist)
{
	g_hash_table_remove(repolist,repo);
}

void gyum_pkglist_disable_repo(gchar *repo,RepoStruct *repostruct,GString *disablerepo)
{
	if (strcmp(repo,"main") && strcmp(repo,"base"))
	{
		gchar *enabled;
		enabled=g_hash_table_lookup(repostruct->vars,"enabled");
		if (!enabled || !strcmp(enabled,"1"))
			g_string_append_printf(disablerepo," --disablerepo=%s",repo);
	}
}

void gyum_pkglist_gchar_free(gchar *repo,gpointer data)
{
	g_free(repo);
}

gchar *gyum_pkglist_make_command(const gchar *extraparams,PkgListOperation operation,gboolean disable_unused_repos)
{
	static const gchar *gyum_pkglist_operation_in_cmd[]={"none","install","update","remove"};
	PkgListForeachData data;
	gchar *cmd=NULL;
	data.command=g_string_new("");
	data.repo=NULL;
	data.operation=gyum_pkglist_operation[operation];
	data.count=0;
	if (extraparams)
		g_string_append_printf(data.command," %s",extraparams);
	g_string_append_printf(data.command," %s",gyum_pkglist_operation_in_cmd[operation]);
	gtk_tree_model_foreach(treeview_list,(GtkTreeModelForeachFunc)gyum_pkglist_foreach_func,&data);
	if (data.count)
	{
		GString *disablerepo=g_string_new("");
		if (disable_unused_repos && (operation!=PKG_LIST_OPERATION_REMOVE))
		{
			GHashTable *repolist;
			repolist=yumrepo_load();
			g_list_foreach(data.repo,(GFunc)gyum_pkglist_check_repo,repolist);
			g_hash_table_foreach(repolist,(GHFunc)gyum_pkglist_disable_repo,disablerepo);
			yumrepo_unload(repolist,FALSE,NULL);
		}
		cmd=g_strdup_printf("yum%s%s",
			disablerepo->str,
			data.command->str);
		g_string_free(disablerepo,TRUE);
	}
	g_string_free(data.command,TRUE);
	g_list_foreach(data.repo,(GFunc)gyum_pkglist_gchar_free,NULL);
	g_list_free(data.repo);
	return cmd;
}

void gyum_controllers_set_sensitive(gboolean sensitive)
{
	static GtkWidget *gyum_handlebox=NULL;
	static GtkWidget *gyum_search_handlebox;
	static GtkWidget *gyum_execute_buttons;
	static GtkWidget *mview;
	static GtkWidget *medit;
	static GtkWidget *mopen;
	static GtkWidget *msave;
	static GtkWidget *fr_operation;
	if (!gyum_handlebox)
	{
		gyum_handlebox=lookup_widget(GTK_WIDGET(treeview),"gyum_handlebox");
		gyum_search_handlebox=lookup_widget(GTK_WIDGET(treeview),"gyum_search_handlebox");
		gyum_execute_buttons=lookup_widget(GTK_WIDGET(treeview),"gyum_execute_buttons");
		mview=lookup_widget(GTK_WIDGET(treeview),"mmview");
		medit=lookup_widget(GTK_WIDGET(treeview),"mmedit");
		mopen=lookup_widget(GTK_WIDGET(treeview),"mopen");
		msave=lookup_widget(GTK_WIDGET(treeview),"msave");
		fr_operation=lookup_widget(GTK_WIDGET(treeview),"fr_operation");
	}
	gtk_widget_set_sensitive(gyum_handlebox,sensitive);
	gtk_widget_set_sensitive(gyum_search_handlebox,sensitive);
	gtk_widget_set_sensitive(gyum_execute_buttons,sensitive);
	gyum_pkglist_op_button_set_sensitive(APPBAR_STATUS_ITEM_SELECT_INSTALL,sensitive);
	gyum_pkglist_op_button_set_sensitive(APPBAR_STATUS_ITEM_SELECT_UPDATE,sensitive);
	gyum_pkglist_op_button_set_sensitive(APPBAR_STATUS_ITEM_SELECT_REMOVE,sensitive);
	gtk_widget_set_sensitive(mview,sensitive);
	gtk_widget_set_sensitive(medit,sensitive);
	gtk_widget_set_sensitive(mopen,sensitive);
	gtk_widget_set_sensitive(msave,sensitive);
	if (!sensitive)
		gtk_widget_set_sensitive(fr_operation,!command_execute & user_root & sensitive);
}
