#include <string.h>
#include <dirent.h>
#include "support.h"
#include "yumrepo.h"
#include "gyum_utils.h"
#include "gyum_preferences.h"

enum {
	REPOSITORY_COLUMN_ID,
	REPOSITORY_COLUMN_NAME,
	REPOSITORY_COLUMN_STATUS,
	REPOSITORY_COLUMN_NUM
};

static GtkWidget *repo_list=NULL;
static GtkTreeModel *repo_list_model=NULL;
static GtkWidget *preferences=NULL;

gboolean gyum_preferences_active()
{
	return preferences!=NULL;
}

static GdkColor not_installed_fg;
static GdkColor installed_fg;
static GdkColor update_fg;
static gchar *fontname=NULL;

void gyum_preferences_repository_row_set(GtkTreeViewColumn *column,GtkCellRenderer *renderer,GtkTreeModel *model,GtkTreeIter *iter,gpointer data)
{
	gchar *status;
	GdkColor *fg;
	gtk_tree_model_get(model,iter,REPOSITORY_COLUMN_STATUS,&status,-1);
	if (!strcmp(status,"enabled")) fg=&not_installed_fg;
	else fg=&installed_fg;
	g_object_set(renderer,"foreground-set",TRUE,"foreground-gdk",fg,NULL);
	if (fontname)
		g_object_set(renderer,"font",fontname,NULL);
}

void gyum_repository_init(GtkWidget *window)
{
	gchar *repository_column_title[]={_("Id"),_("Name"),_("Status")};
	gint i;
	GtkTreeModel *sort_model;
	repo_list=lookup_widget(window,"repo_list");
	repo_list_model=
		GTK_TREE_MODEL(gtk_list_store_new(REPOSITORY_COLUMN_NUM,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING));
	sort_model=gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(repo_list_model));
	gtk_tree_view_set_model(GTK_TREE_VIEW(repo_list),sort_model);
	for (i=0;i<REPOSITORY_COLUMN_NUM;i++)
	{
		GtkCellRenderer *renderer;
		GtkTreeViewColumn *column;
		renderer=gtk_cell_renderer_text_new();
		column=gtk_tree_view_column_new_with_attributes(
			repository_column_title[i],
			renderer,
			"text",i,NULL);
		gtk_tree_view_column_set_resizable(column,TRUE);
		gtk_tree_view_append_column(GTK_TREE_VIEW(repo_list),column);
		gtk_tree_view_column_set_cell_data_func(column,renderer,
			(GtkTreeCellDataFunc)gyum_preferences_repository_row_set,NULL,NULL);
	}
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sort_model),REPOSITORY_COLUMN_NAME,GTK_SORT_ASCENDING);
}

void gyum_config_fill(const gchar *var,const gchar *value,gpointer data)
{
	gchar *widget_name=g_strdup_printf("yumconf_%s",var);
	GtkWidget *widget=lookup_widget(repo_list,widget_name);
	if (widget)
	{
		if (GTK_IS_SPIN_BUTTON(widget))
		{
			gtk_spin_button_set_value(
				GTK_SPIN_BUTTON(widget),
				(gdouble)atoi(value));
		}
		else
		if (GTK_IS_ENTRY(widget))
		{
			gtk_entry_set_text(GTK_ENTRY(widget),value);
		}
		else
		if (GTK_IS_TEXT_VIEW(widget))
		{
			GtkTextBuffer *textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
			gtk_text_buffer_set_text(textbuffer,value,-1);
		}
		else
		if (GTK_IS_CHECK_BUTTON(widget))
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(widget),
				!strcmp(value,"1") ? TRUE:FALSE);
		}
		else
		if (GNOME_IS_FILE_ENTRY(widget))
		{
			gnome_file_entry_set_filename(GNOME_FILE_ENTRY(widget),value);
		}
	}
	g_free(widget_name);
}

void gyum_config_replace_char(gchar *str,gchar old,gchar new)
{
	gchar *ptr;
	while ((ptr=strchr(str,old))!=NULL) *ptr=new;
}

void gyum_config_set_repo(RepoStruct *rs)
{
	static const gchar *yumconf_widget_names[]={
		"cachedir","reposdir","logfile","proxy","proxy_username","proxy_password",
		"distroverpkg","bandwidth","throttle","debuglevel","errorlevel","gpgcheck",
		"assumeyes","tolerant","exactarch","diskspacecheck","recent","retries",
		"exclude","installonlypkgs",NULL};
	int i;
	GtkWidget *widget;
	gchar widget_name[64]="yumconf_";
	if (rs)
	for (i=0;yumconf_widget_names[i];i++)
	{
		strcpy(widget_name+8,yumconf_widget_names[i]);
		widget=lookup_widget(repo_list,widget_name);
		if (widget)
		{
			if (GTK_IS_SPIN_BUTTON(widget))
			{
				gchar *val;
				val=g_strdup_printf("%i",
					gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)));
				yumrepo_struct_set_variable(rs,yumconf_widget_names[i],val);
				g_free(val);
			}
			else
			if (GTK_IS_ENTRY(widget))
			{
				const gchar *val=gtk_entry_get_text(GTK_ENTRY(widget));
				if (val && *val)
					yumrepo_struct_set_variable(rs,yumconf_widget_names[i],val);
			}
			else
			if (GTK_IS_TEXT_VIEW(widget))
			{
				GtkTextBuffer *textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
				GtkTextIter start,end;
				gchar *val;
				gtk_text_buffer_get_start_iter(textbuffer,&start);
				gtk_text_buffer_get_end_iter(textbuffer,&end);
				val=gtk_text_buffer_get_text(textbuffer,&start,&end,FALSE);
				if (val && *val)
				{
					gyum_config_replace_char(val,'\n',' ');
					yumrepo_struct_set_variable(rs,yumconf_widget_names[i],val);
				}
				g_free(val);
			}
			else
			if (GTK_IS_CHECK_BUTTON(widget))
			{
				yumrepo_struct_set_variable(rs,yumconf_widget_names[i],
					gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) ? "1":"0");
			}
			else
			if (GNOME_IS_FILE_ENTRY(widget))
			{
				gchar *val;
				val=gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(widget),TRUE);
				if (val)
				{
					yumrepo_struct_set_variable(rs,yumconf_widget_names[i],val);
					g_free(val);
				}
			}	
		}
	}
	else
		g_warning("YUM config record not found!");
}

static GHashTable *yum_repo_list=NULL;

void gyum_repository_fill(const gchar *id,RepoStruct *repo_struct,gpointer data)
{
	if (!repo_struct->remove)
	{
		gyum_config_get_color("gui_not_installed_color",&not_installed_fg);
		gyum_config_get_color("gui_installed_color",&installed_fg);
		gyum_config_get_color("gui_update_color",&update_fg);
		if (fontname) g_free(fontname);
		fontname=gyum_config_get_string("list_font");
		if (strcmp(id,"main"))
		{
			GtkTreeIter iter;
			const gchar *name;
			const gchar *status;
			name=g_hash_table_lookup(repo_struct->vars,"name");
			status=g_hash_table_lookup(repo_struct->vars,"enabled");
			gtk_list_store_append(GTK_LIST_STORE(repo_list_model),&iter);
			if (status && !strcmp(status,"0"))
				status="disabled";
			else
				status="enabled";
			gtk_list_store_set(GTK_LIST_STORE(repo_list_model),&iter,
				REPOSITORY_COLUMN_ID,id,
				REPOSITORY_COLUMN_NAME,name,
				REPOSITORY_COLUMN_STATUS,status,
				-1);
		}
		else
		{
			g_hash_table_foreach(repo_struct->vars,(GHFunc)gyum_config_fill,NULL);
		}
	}
}

void gyum_repository_data_clear()
{
	static const gchar *widget_repo_name[]={
		"repo_enabled","repo_id","repo_name","repo_baseurl","repo_mirrorlist",
		"repo_gpgcheck","repo_gpgkey",NULL};
	gint i;
	for (i=0;widget_repo_name[i];i++)
	{
		GtkWidget *widget=lookup_widget(repo_list,widget_repo_name[i]);
		if (GTK_IS_CHECK_BUTTON(widget))
		{
			gboolean check=!strcmp(widget_repo_name[i],"repo_enabled");
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),check);
		}
		else
		if (GTK_IS_ENTRY(widget))
		{
			gtk_entry_set_text(GTK_ENTRY(widget),"");
		}
	}
}

static gboolean gyum_repository_status_fill=FALSE;

void gyum_repository_data_fill(const gchar *var,const gchar *val,gpointer data)
{
	if (*var!='#')
	{
		gchar *widget_name=g_strdup_printf("repo_%s",var);
		GtkWidget *widget=lookup_widget(repo_list,widget_name);
		if (GTK_IS_CHECK_BUTTON(widget))
		{
			gboolean check=!strcmp(val,"1");
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),check);
		}
		else
		if (GTK_IS_ENTRY(widget))
		{
			gtk_entry_set_text(GTK_ENTRY(widget),val);
		}
		g_free(widget_name);
	}
}

void gyum_repository_cursor_changed()
{
	GtkTreePath *path;
	GtkTreePath *child_path;
	GtkTreeViewColumn *column;
	GtkTreeIter iter;
	gchar *id;
	RepoStruct *repo_struct;
	gyum_repository_status_fill=TRUE;
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(repo_list),&path,&column);
	child_path=gtk_tree_model_sort_convert_path_to_child_path(
		GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(repo_list))),path);
	gtk_tree_model_get_iter(repo_list_model,&iter,child_path);
	gtk_tree_model_get(repo_list_model,&iter,REPOSITORY_COLUMN_ID,&id,-1);
	repo_struct=g_hash_table_lookup(yum_repo_list,id);
	gyum_repository_data_clear();
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(repo_list,"repo_id")),id);
	if (repo_struct)
		g_hash_table_foreach(repo_struct->vars,(GHFunc)gyum_repository_data_fill,NULL);
	gtk_label_set_text(GTK_LABEL(lookup_widget(repo_list,"repo_filename")),repo_struct->filename);
	g_free(id);
	gyum_repository_status_fill=FALSE;
	gtk_tree_path_free(path);
	gtk_tree_path_free(child_path);
}

void gyum_repository_new()
{
	gyum_repository_status_fill=TRUE;
	gyum_repository_data_clear();
	gyum_repository_status_fill=FALSE;
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(GTK_WIDGET(repo_list),"repo_id")),
		"<new repository>");
}

void gyum_repository_import(GtkWidget *gyum_repo_import)
{
	GtkTextBuffer *textbuffer;
	GtkTextIter start,end;
	gchar *str;
	textbuffer=gtk_text_view_get_buffer(
		GTK_TEXT_VIEW(lookup_widget(gyum_repo_import,"gyum_repo_import_text")));
	gtk_text_buffer_get_start_iter(textbuffer,&start);
	gtk_text_buffer_get_end_iter(textbuffer,&end);
	str=gtk_text_buffer_get_text(textbuffer,&start,&end,FALSE);
	yumrepo_load_text(yum_repo_list,str);
	g_free(str);
	gtk_list_store_clear(GTK_LIST_STORE(repo_list_model));
	g_hash_table_foreach(yum_repo_list,(GHFunc)gyum_repository_fill,NULL);
}

void gyum_repository_edit(const gchar *var,const gchar *val)
{
	if (!gyum_repository_status_fill)
	{
		GtkTreePath *path=NULL;
		GtkTreePath *child_path=NULL;
		GtkTreeViewColumn *column;
		GtkTreeIter iter;
		gchar *id=NULL;
		RepoStruct *repo_struct=NULL;
		GtkTreeModel *sort_model;
		sort_model=gtk_tree_view_get_model(GTK_TREE_VIEW(repo_list));
		if (strcmp(var,"id") || strcmp(val,"<new repository>"))
			gtk_tree_view_get_cursor(GTK_TREE_VIEW(repo_list),&path,&column);
		if (path)
		{
			child_path=
				gtk_tree_model_sort_convert_path_to_child_path(GTK_TREE_MODEL_SORT(sort_model),path);
			gtk_tree_model_get_iter(repo_list_model,&iter,child_path);
			gtk_tree_model_get(repo_list_model,&iter,REPOSITORY_COLUMN_ID,&id,-1);
			if (id) repo_struct=g_hash_table_lookup(yum_repo_list,id);
		}
		if (!strcmp(var,"id"))
		{
			RepoStruct *new_repo_struct=yumrepo_struct_alloc();
			if (repo_struct)
			{
				yumrepo_struct_copy(repo_struct,new_repo_struct);
				repo_struct->remove=TRUE;
				new_repo_struct->change=TRUE;
				if (access(repo_struct->filename,F_OK))
				{
					new_repo_struct->filename=yumrepo_filename(val);
					gtk_label_set_text(GTK_LABEL(lookup_widget(repo_list,"repo_filename")),
											new_repo_struct->filename);
				}
				g_hash_table_insert(yum_repo_list,g_strdup(val),new_repo_struct);
				gtk_list_store_set(GTK_LIST_STORE(repo_list_model),&iter,
					REPOSITORY_COLUMN_ID,val,-1);
			}
			else
			{
				new_repo_struct->change=TRUE;
				yumrepo_struct_set_id(new_repo_struct,val);
				g_hash_table_insert(yum_repo_list,g_strdup(val),new_repo_struct);
				gtk_list_store_append(GTK_LIST_STORE(repo_list_model),&iter);
				gtk_list_store_set(GTK_LIST_STORE(repo_list_model),&iter,
					REPOSITORY_COLUMN_ID,val,
					REPOSITORY_COLUMN_STATUS,"enabled",-1);
				child_path=gtk_tree_model_get_path(repo_list_model,&iter);
				path=gtk_tree_model_sort_convert_child_path_to_path(GTK_TREE_MODEL_SORT(sort_model),child_path);
				gtk_tree_view_set_cursor(GTK_TREE_VIEW(repo_list),
					path,
					NULL,FALSE);
			}
			repo_struct=new_repo_struct;
		}
		else
		{
			if (repo_struct)
			{
    			    if (!strcmp(var,"name"))
			    {
			    	    gtk_list_store_set(GTK_LIST_STORE(repo_list_model),&iter,
					    REPOSITORY_COLUMN_NAME,val,-1);
			    }
			    else
			    if (!strcmp(var,"enabled"))
			    {
				    gtk_list_store_set(GTK_LIST_STORE(repo_list_model),&iter,
					    REPOSITORY_COLUMN_STATUS,(!strcmp(val,"1") ? "enabled":"disabled"),-1);
			    }
			    yumrepo_struct_set_variable(repo_struct,var,val);
			}
		}
		if (path) gtk_tree_path_free(path);
		if (child_path) gtk_tree_path_free(child_path);
		if (id) g_free(id);
	}
}

void gyum_repository_remove()
{
	GtkTreePath *path;
	GtkTreeViewColumn *column;
	GtkTreeIter iter;
	gchar *id;
	RepoStruct *repo;
	gyum_repository_status_fill=TRUE;
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(repo_list),&path,&column);
	gtk_tree_model_get_iter(repo_list_model,&iter,path);
	gtk_tree_model_get(repo_list_model,&iter,REPOSITORY_COLUMN_ID,&id,-1);
	repo=g_hash_table_lookup(yum_repo_list,id);
	repo->remove=TRUE;
	gtk_list_store_remove(GTK_LIST_STORE(repo_list_model),&iter);
	gyum_repository_status_fill=TRUE;
	gyum_repository_data_clear();
	gyum_repository_status_fill=FALSE;
	g_free(id);
}


void gyum_if_config_init(GtkWidget *window)
{
	GtkWidget *treeview;
	GtkListStore *model;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *button;
	treeview=lookup_widget(window,"ifconf_binary_paths");
	renderer=gtk_cell_renderer_text_new();
	column=gtk_tree_view_column_new_with_attributes(_("Path"),renderer,"text",0,NULL);
	model=gtk_list_store_new(1,G_TYPE_STRING);
	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview),GTK_TREE_MODEL(model));
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview),column);
	
	treeview=lookup_widget(window,"ifconf_library_paths");
	renderer=gtk_cell_renderer_text_new();
	column=gtk_tree_view_column_new_with_attributes(_("Path"),renderer,"text",0,NULL);
	model=gtk_list_store_new(1,G_TYPE_STRING);
	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview),GTK_TREE_MODEL(model));
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview),column);
	
	button=lookup_widget(GTK_WIDGET(treeview),"ifconf_binary_path_remove");
	gtk_widget_set_sensitive(button,FALSE);
	button=lookup_widget(GTK_WIDGET(treeview),"ifconf_binary_path_add");
	gtk_widget_set_sensitive(button,FALSE);
	
	button=lookup_widget(GTK_WIDGET(treeview),"ifconf_library_path_remove");
	gtk_widget_set_sensitive(button,FALSE);
	button=lookup_widget(GTK_WIDGET(treeview),"ifconf_library_path_add");
	gtk_widget_set_sensitive(button,FALSE);
}

void gyum_if_config_fill_tree(gchar *str,GtkListStore *list)
{
	GtkTreeIter iter;
	gtk_list_store_append(list,&iter);
	gtk_list_store_set(list,&iter,0,str,-1);
}

void gyum_if_config_fill(GtkWidget *window)
{
	GtkWidget *terminal_background;
	GtkWidget *terminal_foreground;
	GtkWidget *gui_color;
	GdkColormap *colormap;
	GdkColor bg;
	GdkColor fg;
	GSList *list;
	GtkTreeModel *model;
	gchar *fontname;
	colormap=gtk_widget_get_colormap(window);
	
	terminal_background=lookup_widget(window,"ifconf_terminal_background");
	gyum_config_get_color("terminal_background",&bg);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(terminal_background),&bg);
	
	terminal_foreground=lookup_widget(window,"ifconf_terminal_foreground");
	gyum_config_get_color("terminal_foreground",&fg);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(terminal_foreground),&fg);
	
	fontname=gyum_config_get_string("terminal_font");
	if (fontname)
	{
		GtkWidget *terminal_font;
		terminal_font=lookup_widget(window,"ifconf_terminal_font");
		gtk_font_button_set_font_name(GTK_FONT_BUTTON(terminal_font),fontname);
		g_free(fontname);
	}
	
	gtk_range_set_value(GTK_RANGE(lookup_widget(window,"ifconf_terminal_priority")),
		(gdouble)gyum_config_get_int("terminal_priority"));
	
	gtk_range_set_value(GTK_RANGE(lookup_widget(window,"ifconf_gui_sensitivity")),
		(gdouble)gyum_config_get_int("gui_sensitivity"));
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(lookup_widget(window,"ifconf_gui_show_splash_window")),
		gyum_config_get_boolean("show_splash_window"));
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(lookup_widget(window,"ifconf_gui_show_notebook")),
		gyum_config_get_boolean("gui_show_notebook"));
	
	fontname=gyum_config_get_string("list_font");
	if (fontname)
	{
		GtkWidget *gui_font;
		gui_font=lookup_widget(window,"ifconf_gui_font");
		gtk_font_button_set_font_name(GTK_FONT_BUTTON(gui_font),fontname);
		g_free(fontname);
	}
	gui_color=lookup_widget(window,"ifconf_gui_not_installed_color");
	gyum_config_get_color("gui_not_installed_color",&fg);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(gui_color),&fg);
	
	gui_color=lookup_widget(window,"ifconf_gui_installed_color");
	gyum_config_get_color("gui_installed_color",&fg);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(gui_color),&fg);
	
	gui_color=lookup_widget(window,"ifconf_gui_update_color");
	gyum_config_get_color("gui_update_color",&fg);
	gtk_color_button_set_color(GTK_COLOR_BUTTON(gui_color),&fg);
	
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(lookup_widget(window,"ifconf_packageinfo_startup_loading")),
		gyum_config_get_boolean("startup_loading"));
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(lookup_widget(window,"ifconf_packageinfo_distributor")),
		gyum_config_get_boolean("distributor_packages"));
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(lookup_widget(window,"ifconf_packageinfo_external")),
		gyum_config_get_boolean("external_packages"));
	
	list=gyum_config_get_list("rpminfo_binary_paths");
	model=gtk_tree_view_get_model(GTK_TREE_VIEW(lookup_widget(window,"ifconf_binary_paths")));
	gtk_list_store_clear(GTK_LIST_STORE(model));
	g_slist_foreach(list,(GFunc)gyum_if_config_fill_tree,model);
	list=gyum_config_get_list("rpminfo_library_paths");
	model=gtk_tree_view_get_model(GTK_TREE_VIEW(lookup_widget(window,"ifconf_library_paths")));
	gtk_list_store_clear(GTK_LIST_STORE(model));
	g_slist_foreach(list,(GFunc)gyum_if_config_fill_tree,model);
}

void gyum_if_config_treeview_cursor_changed(GtkTreeView *treeview,const gchar *type)
{
	GtkTreePath *path;
	gtk_tree_view_get_cursor(treeview,&path,NULL);
	if (path)
	{
		GtkTreeModel *model;
		GtkTreeIter iter;
		gchar *path_str;
		model=gtk_tree_view_get_model(treeview);
		gtk_tree_model_get_iter(model,&iter,path);
		gtk_tree_model_get(model,&iter,0,&path_str,-1);
		if (path_str)
		{
			GtkWidget *fileentry;
			gchar *fileentry_name;
			GtkWidget *button;
			gchar *button_name;
			fileentry_name=g_strdup_printf("ifconf_select_%s_path",type);
			fileentry=lookup_widget(GTK_WIDGET(treeview),fileentry_name);
			gnome_file_entry_set_filename(GNOME_FILE_ENTRY(fileentry),path_str);
			g_free(fileentry_name);
			
			button_name=g_strdup_printf("ifconf_%s_path_remove",type);
			button=lookup_widget(GTK_WIDGET(treeview),fileentry_name);
			gtk_widget_set_sensitive(button,TRUE);
			g_free(button_name);
			
			button_name=g_strdup_printf("ifconf_%s_path_add",type);
			button=lookup_widget(GTK_WIDGET(treeview),fileentry_name);
			gtk_widget_set_sensitive(button,FALSE);
			g_free(button_name);
		}
		gtk_tree_path_free(path);
	}
}

void gyum_if_config_fileentry_changed(GnomeFileEntry *fileentry,const gchar *type)
{
	gchar *path;
	GtkWidget *remove_button;
	gchar *remove_button_name;
	GtkWidget *add_button;
	gchar *add_button_name;
	path=gnome_file_entry_get_full_path(fileentry,TRUE);
	remove_button_name=g_strdup_printf("ifconf_%s_path_remove",type);
	remove_button=lookup_widget(GTK_WIDGET(fileentry),remove_button_name);
	g_free(remove_button_name);
	add_button_name=g_strdup_printf("ifconf_%s_path_add",type);
	add_button=lookup_widget(GTK_WIDGET(fileentry),add_button_name);
	g_free(add_button_name);
	if (path)
	{
		gtk_widget_set_sensitive(remove_button,FALSE);
		gtk_widget_set_sensitive(add_button,TRUE);
		g_free(path);
	}
	else
	{
		gtk_widget_set_sensitive(remove_button,FALSE);
		gtk_widget_set_sensitive(add_button,FALSE);
	}
}

void gyum_if_config_treeview_item_op(GtkWidget *button,const gchar *type,gboolean add)
{
	GtkTreeView *treeview;
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *name;
	name=g_strdup_printf("ifconf_%s_paths",type);
	treeview=GTK_TREE_VIEW(lookup_widget(button,name));
	g_free(name);
	store=GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
	if (add)
	{
		GnomeFileEntry *fileentry;
		gchar *path;
		name=g_strdup_printf("ifconf_select_%s_path",type);
		fileentry=GNOME_FILE_ENTRY(lookup_widget(button,name));
		g_free(name);
		path=gnome_file_entry_get_full_path(fileentry,TRUE);
		if (path)
		{
			gtk_list_store_append(store,&iter);
			gtk_list_store_set(store,&iter,0,path,-1);
			g_free(path);
		}
	}
	else
	{		
		GtkTreePath *path;
		gtk_tree_view_get_cursor(treeview,&path,NULL);
		gtk_tree_model_get_iter(GTK_TREE_MODEL(store),&iter,path);
		gtk_tree_path_free(path);
		gtk_list_store_remove(store,&iter);
	}
	gtk_widget_set_sensitive(button,FALSE);
}

gboolean gyum_if_config_treeview2slist(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,GSList **list)
{
	gchar *path_str;
	gtk_tree_model_get(model,iter,0,&path_str,-1);
	*list=g_slist_append(*list,path_str);
	return FALSE;
}

void gyum_slist_free(gpointer ptr,gpointer data)
{
	g_free(ptr);
}

void gyum_if_config_set()
{
	GtkWidget *treeview;
	GtkTreeModel *model;
	GSList *list;
	treeview=lookup_widget(repo_list,"ifconf_binary_paths");
	model=gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	list=NULL;
	gtk_tree_model_foreach(model,(GtkTreeModelForeachFunc)gyum_if_config_treeview2slist,&list);
	gyum_config_set_list("rpminfo_binary_paths",list);
	treeview=lookup_widget(repo_list,"ifconf_library_paths");
	model=gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	g_slist_foreach(list,(GFunc)gyum_slist_free,NULL);
	g_slist_free(list);
	list=NULL;
	gtk_tree_model_foreach(model,(GtkTreeModelForeachFunc)gyum_if_config_treeview2slist,&list);
	gyum_config_set_list("rpminfo_library_paths",list);
	g_slist_foreach(list,(GFunc)gyum_slist_free,NULL);
	g_slist_free(list);
}


void gyum_preferences_init(GtkWidget *window)
{
	preferences=window;
	gyum_repository_init(window);
	
	yum_repo_list=yumrepo_load();
	g_hash_table_foreach(yum_repo_list,(GHFunc)gyum_repository_fill,NULL);
		
	gyum_if_config_init(window);
	gyum_if_config_fill(window);
}

void gyum_preferences_unload_variables(gboolean update);

gboolean gyum_preferences_unload(GtkWidget *window,gboolean update,gboolean *changed)
{
	gboolean successful=TRUE;
	*changed=FALSE;
	if (update)
	{
		RepoStruct *rs;
		rs=g_hash_table_lookup(yum_repo_list,"main");
		gyum_config_set_repo(rs);
		gyum_if_config_set();
	}
	gyum_preferences_unload_variables(update);
	successful&=yumrepo_unload(yum_repo_list,update,changed);
	if (successful)
	{
		g_hash_table_destroy(yum_repo_list);
		yum_repo_list=NULL;
		preferences=NULL;
	}
	return successful;
}

static GHashTable *gyum_preferences_changed_vars=NULL;

void gyum_preferences_set_variable(const gchar *var,GConfValue *val)
{
	if (!gyum_preferences_changed_vars)
		gyum_preferences_changed_vars=g_hash_table_new_full(g_str_hash,g_str_equal,g_free,(GDestroyNotify)gconf_value_free);
	g_hash_table_replace(gyum_preferences_changed_vars,g_strdup(var),val);
}

void gyum_preferences_unload_variable(const gchar *var,GConfValue *val,gpointer data)
{
	switch (val->type)
	{
		case GCONF_VALUE_STRING:
			gyum_config_set_string(var,gconf_value_get_string(val));
			break;
		case GCONF_VALUE_BOOL:
			gyum_config_set_boolean(var,gconf_value_get_bool(val));
			break;
		case GCONF_VALUE_INT:
			gyum_config_set_int(var,gconf_value_get_int(val));
			break;
		case GCONF_VALUE_LIST:
			gyum_config_set_list(var,gconf_value_get_list(val));
			break;
		case GCONF_VALUE_INVALID:
		case GCONF_VALUE_SCHEMA:
		case GCONF_VALUE_FLOAT:
		case GCONF_VALUE_PAIR:
			// not used
			break;
	}
}

void gyum_preferences_unload_variables(gboolean update)
{
	if (update)
		g_hash_table_foreach(gyum_preferences_changed_vars,(GHFunc)gyum_preferences_unload_variable,NULL);
	g_hash_table_destroy(gyum_preferences_changed_vars);
	gyum_preferences_changed_vars=NULL;
}
