#include "gyum_cache.h"
#include "interface.h"
#include "support.h"
#include "gyum_utils.h"
#include "gyum_rpm.h"
#include "gyum_pkglist.h"
#include "gyum_config.h"

static GtkWidget *gyum_execute_rpm=NULL;
static GtkWidget *gyum_rpm_packages=NULL;

void gyum_rpm_packages_append(gchar *package,GtkListStore *store)
{
	GtkTreeIter iter;
	gchar *dir;
	gchar *base;
	dir=g_path_get_dirname(package);
	base=g_path_get_basename(package);
	gtk_list_store_append(store,&iter);
	gtk_list_store_set(store,&iter,0,base,1,dir,-1);
	g_free(package);
	g_free(dir);
	g_free(base);
}

void gyum_rpm_show()
{
	if (!gyum_execute_rpm)
	{
		GList *packages;
		GtkListStore *store;
		GtkTreeModel *sort_model;
		GtkTreeViewColumn *column;
		GtkCellRenderer *renderer;
		gyum_execute_rpm=create_gyum_execute_rpm();
	
		gyum_rpm_packages=lookup_widget(gyum_execute_rpm,"gyum_rpm_packages");
		store=gtk_list_store_new(2,G_TYPE_STRING,G_TYPE_STRING);
		sort_model=gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
		gtk_tree_view_set_model(GTK_TREE_VIEW(gyum_rpm_packages),sort_model);
		
		renderer=gtk_cell_renderer_text_new();
		column=gtk_tree_view_column_new_with_attributes(_("Package"),renderer,"text",0,NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(gyum_rpm_packages),column);
		renderer=gtk_cell_renderer_text_new();
		column=gtk_tree_view_column_new_with_attributes(_("Directory"),renderer,"text",1,NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(gyum_rpm_packages),column);
		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sort_model),0,GTK_SORT_ASCENDING);
	
		packages=gyum_cache_pkgs_selected();
		g_list_foreach(packages,(GFunc)gyum_rpm_packages_append,store);
		g_list_free(packages);
	
		gtk_widget_show(gyum_execute_rpm);
	}
}

void gyum_rpm_destroy()
{
	gtk_widget_destroy(gyum_execute_rpm);
	gyum_execute_rpm=NULL;
}

void gyum_rpm_command_page_prepare()
{
	static gchar *gyum_rpm_operation[]={"install","upgrade","freshen",NULL};
	static gchar *gyum_rpm_flags[]={
		"test",
		"force","nodeps","replacepkgs","replacefiles",
		"noscripts","nopre","nopost","nopreun","nopostun",NULL};
	static gchar *gyum_rpm_path[]={"root","dbpath",NULL};
	GString *param;
	gchar *path;
	int i;
	param=g_string_new("-hv");
	for (i=0;gyum_rpm_operation[i];i++)
	{
		gchar *name;
		name=g_strdup_printf("gyum_rpm_%s",gyum_rpm_operation[i]);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
				lookup_widget(gyum_execute_rpm,name))))
		{
			g_string_append_printf(param," --%s",gyum_rpm_operation[i]);
			g_free(name);
			break;
		}
		g_free(name);
	}
	for (i=0;gyum_rpm_flags[i];i++)
	{
		gchar *name;
		name=g_strdup_printf("gyum_rpm_flag_%s",gyum_rpm_flags[i]);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
				lookup_widget(gyum_execute_rpm,name))))
		{
			g_string_append_printf(param," --%s",gyum_rpm_flags[i]);
		}
		g_free(name);
	}
	for (i=0;gyum_rpm_path[i];i++)
	{
		gchar *name;
		name=g_strdup_printf("gyum_rpm_path_%s",gyum_rpm_path[i]);
		path=gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(
				lookup_widget(gyum_execute_rpm,name)),TRUE);
		if (path)
		{
			g_string_append_printf(param," --%s %s",gyum_rpm_path[i],path);
			g_free(path);
		}
		g_free(name);
	}
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(gyum_execute_rpm,"gyum_rpm_params")),param->str);
}

void gyum_rpm_file_remove()
{
	GtkTreePath *path;
	GtkTreeIter iter;
	GtkTreeModel *sort_model;
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(gyum_rpm_packages),&path,NULL);
	sort_model=gtk_tree_view_get_model(GTK_TREE_VIEW(gyum_rpm_packages));
	if (gtk_tree_model_get_iter(sort_model,&iter,path))
	{
		GtkTreeModel *model;
		GtkTreeIter child_iter;
		model=gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(sort_model));
		gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT(sort_model),
				&child_iter,&iter);
		gtk_list_store_remove(GTK_LIST_STORE(model),&child_iter);
	}
	gtk_tree_path_free(path);
}

void gyum_rpm_package_add(gchar *package,GtkListStore *store)
{
	gyum_rpm_packages_append(package,store);
}

void gyum_rpm_file_add()
{
	GtkWidget *filechooser;
	GtkFileFilter *filter;
	gchar *rpm_directory;
	gint ret;
	GSList *files=NULL;
	gchar *dir;
	filechooser=gtk_file_chooser_dialog_new(_("Select packages"),NULL,
					GTK_FILE_CHOOSER_ACTION_OPEN,
					GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN,GTK_RESPONSE_OK,NULL);
	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(filechooser),TRUE);
	filter=gtk_file_filter_new();
	gtk_file_filter_set_name(filter,"RPM packages");
	gtk_file_filter_add_pattern(filter,"*.rpm");
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filechooser),filter);
	rpm_directory=gyum_config_get_string("rpm_directory");
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filechooser),rpm_directory);
	g_free(rpm_directory);
	ret=gtk_dialog_run(GTK_DIALOG(filechooser));
	switch (ret)
	{
		case GTK_RESPONSE_OK:
			files=gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(filechooser));
			dir=gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(filechooser));
			gyum_config_set_string("rpm_directory",dir);
			g_free(dir);
			break;
	}
	gtk_widget_destroy(filechooser);
	if (files)
	{
		GtkTreeModel *sort_model;
		GtkTreeModel *model;
		sort_model=gtk_tree_view_get_model(GTK_TREE_VIEW(gyum_rpm_packages));
		model=gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(sort_model));
		g_slist_foreach(files,(GFunc)gyum_rpm_package_add,model);
		g_slist_free(files);
	}
}

gboolean gyum_rpm_files(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,GString *files)
{
	gchar *dir;
	gchar *base;
	gchar *file;
	gtk_tree_model_get(model,iter,0,&base,1,&dir,-1);
	file=g_strdup_printf("%s/%s",dir,base);
	g_string_append_printf(files," %s",file);
	g_free(file);
	g_free(dir);
	g_free(base);
	return FALSE;
}

void gyum_rpm_execute_done(pid_t pid,gint status,gpointer data)
{
	if (!status)
	{
		gyum_rpm_destroy();
		gyum_pkglist_load();
	}
	else
	{
		GtkWidget *gyum_rpm_druid;
		gtk_widget_set_sensitive(gyum_execute_rpm,TRUE);
		gtk_widget_hide(gyum_execute_rpm);
		gtk_window_present(GTK_WINDOW(gyum_execute_rpm));
		gyum_rpm_druid=lookup_widget(gyum_execute_rpm,"gyum_rpm_druid");
		gnome_druid_set_page(
			GNOME_DRUID(gyum_rpm_druid),
			GNOME_DRUID_PAGE(lookup_widget(gyum_execute_rpm,"gyum_rpm_druidpage_command")));
	}
}

void gyum_rpm_command_execute(GtkWidget *notebook)
{
	const gchar *params;
	GString *files;
	gchar *command;
	GtkTreeModel *sort_model;
	params=gtk_entry_get_text(GTK_ENTRY(lookup_widget(gyum_execute_rpm,"gyum_rpm_params")));
	files=g_string_new("");
	sort_model=gtk_tree_view_get_model(GTK_TREE_VIEW(gyum_rpm_packages));
	gtk_tree_model_foreach(gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(sort_model)),
		(GtkTreeModelForeachFunc)gyum_rpm_files,files);
	if (*files->str)
	{
		command=g_strdup_printf("rpm %s %s",params,files->str);
		gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook),1);
		//gtk_widget_set_sensitive(gyum_execute_rpm,FALSE);
		gtk_widget_hide(gyum_execute_rpm);
		gyum_command_execute(_("Executing RPM command"),command,TRUE,(GECFunc)gyum_rpm_execute_done,NULL);
		g_free(command);
	}
	g_string_free(files,TRUE);
}
