#include <vte/vte.h>
#define __USE_GNU
#include <unistd.h>
#include <sys/wait.h>
#include <vte/reaper.h>
#include "interface.h"
#include "support.h"
#include "gyum_config.h"
#include "gyum_utils.h"
#include "splash.xbm"

static GtkWidget *gyum_splash=NULL;

void splash_hide()
{
	if (gyum_splash)
	{
		gtk_widget_destroy(gyum_splash);
		gyum_splash=NULL;
		while (gtk_events_pending()) gtk_main_iteration();
	}
}

gboolean timeout_splash_hide(gpointer data)
{
	splash_hide();
	return FALSE;
}

void splash_show()
{
	if (gyum_config_get_boolean("show_splash_window"))
	{
		GtkWidget *gyum_splash_image;
		GdkPixbuf *pixbuf;
		gint width;
		gint height;
		GdkBitmap *bitmap;
		pixbuf=create_pixbuf("gnome-yum/splash.png");
		width=gdk_pixbuf_get_width(pixbuf);
		height=gdk_pixbuf_get_height(pixbuf);
		gyum_splash=create_gyum_splash();
		gtk_window_resize(GTK_WINDOW(gyum_splash),width,height);
		gyum_splash_image=lookup_widget(gyum_splash,"gyum_splash_image");
		gtk_image_set_from_pixbuf(GTK_IMAGE(gyum_splash_image),pixbuf);
		bitmap=gdk_bitmap_create_from_data(NULL,(gchar *)gnome_yum_logo_bits,gnome_yum_logo_width,gnome_yum_logo_height);
		gtk_widget_shape_combine_mask(
			gyum_splash,
			bitmap,
			0,0);
		gtk_widget_show_now(gyum_splash);
		gdk_window_process_all_updates();
		while (gtk_events_pending()) gtk_main_iteration();
		g_timeout_add(5000,(GSourceFunc)timeout_splash_hide,NULL);
	}
}

static GtkWidget *terminal=NULL;
static VteReaper *reaper=NULL;
static GtkWidget *messages=NULL;
typedef struct {
	pid_t pid;
	GECFunc func;
	gpointer user_data;
	GTimer *timer;
} GYUMTerminalData;
static GYUMTerminalData gyumutils_data={-1,NULL,NULL,NULL};

static gboolean gyumutils_terminal_busy=FALSE;

void gyum_terminal_child_exited(VteTerminal *terminal,pid_t pid,gint status,GYUMTerminalData *data)
{
	if (data->pid!=-1)
	{
		gulong elapsed;
		gdouble sec;
		g_timer_stop(data->timer);
		sec=g_timer_elapsed(data->timer,&elapsed);
		g_timer_destroy(data->timer);
		gtk_widget_set_sensitive(lookup_widget(messages,"bt_terminal_stop"),FALSE);
		if (!status)
		{
			gchar *str;
			str=g_strdup_printf(_("... successful. (elapsed: %.0f sec)\n"),sec);
			gyum_insert_message(NULL,str);
			g_free(str);
		}
		else
		{
			gchar *str;
			str=g_strdup_printf(_("... unsuccessful. (elapsed: %.0f sec)\n"),sec);
			gyum_insert_message(NULL,str);
			g_free(str);
		}
		gyumutils_terminal_busy=FALSE;
		if (data->func)
			data->func(data->pid,status,data->user_data);
	}
}

void gyum_create_terminal(GtkWidget *window)
{
	GtkWidget *vbox_terminal;
	GdkColor bg;
	GdkColor fg;
	GdkColormap *colormap;
	gchar *fontname;
	vbox_terminal=lookup_widget(window,"vbox_terminal");
	terminal=vte_terminal_new();
	vte_terminal_set_background_transparent(VTE_TERMINAL(terminal),FALSE);
	colormap=gtk_widget_get_colormap(window);
	gdk_colormap_alloc_color(colormap,&bg,TRUE,TRUE);
	gyum_config_get_color("terminal_foreground",&fg);
	gdk_colormap_alloc_color(colormap,&fg,TRUE,TRUE);
	gyum_config_get_color("terminal_background",&bg);
	vte_terminal_set_colors(VTE_TERMINAL(terminal),&fg,&bg,NULL,0);
	fontname=gyum_config_get_string("terminal_font");
	if (fontname)
	{
		vte_terminal_set_font_from_string(VTE_TERMINAL(terminal),fontname);
		g_free(fontname);
	}
	reaper=vte_reaper_get();
	gtk_box_pack_start(GTK_BOX(vbox_terminal),terminal,TRUE,TRUE,4);
	gtk_widget_show(terminal);
	messages=lookup_widget(window,"gyum_messages");
	g_signal_connect(G_OBJECT(reaper),"child-exited",
				(GCallback)gyum_terminal_child_exited,&gyumutils_data);
}

void gyum_terminal_set_color(GYUMTerminalColor which,const GdkColor *color)
{
	const gchar *key=NULL;
	switch (which)
	{
		case GYUM_TERMINAL_COLOR_BACKGROUND:
			vte_terminal_set_color_background(VTE_TERMINAL(terminal),color);
			key="terminal_background";
			break;
		case GYUM_TERMINAL_COLOR_FOREGROUND:
			vte_terminal_set_color_foreground(VTE_TERMINAL(terminal),color);
			key="terminal_foreground";
			break;
	}
	if (key) gyum_config_set_color(key,color);
}

void gyum_terminal_set_font(const gchar *fontname)
{
	vte_terminal_set_font_from_string(VTE_TERMINAL(terminal),fontname);
	gyum_config_set_string("terminal_font",fontname);
}

gboolean gyum_command_execute(const gchar *msg,const gchar *command,gboolean with_terminal,
		GECFunc func,gpointer user_data)
{
	gint exit_status=0;
	gchar *output=NULL;
	gchar *error=NULL;
	gyum_insert_message(msg,command);
	if (with_terminal)
	{
		if (!gyumutils_terminal_busy)
		{
			gchar **argv=g_strsplit(command," ",0);
			gint priority;
			gtk_widget_grab_focus(terminal);
			gyumutils_terminal_busy=TRUE;
			gyumutils_data.func=func;
			gyumutils_data.user_data=user_data;
			gyumutils_data.timer=g_timer_new();
			g_timer_start(gyumutils_data.timer);
			gtk_widget_set_sensitive(lookup_widget(terminal,"bt_terminal_stop"),TRUE);
			gyumutils_data.pid=vte_terminal_fork_command(VTE_TERMINAL(terminal),argv[0],argv,environ,NULL,FALSE,FALSE,FALSE);
			priority=gyum_config_get_int("terminal_priority");
			setpriority(PRIO_PROCESS,gyumutils_data.pid,priority);
			vte_reaper_add_child(gyumutils_data.pid);
			g_strfreev(argv);
		}
		else
		{
			GtkWidget *warning;
			warning=gtk_message_dialog_new(NULL,
				GTK_DIALOG_MODAL,
				GTK_MESSAGE_WARNING,
				GTK_BUTTONS_OK,
				_("Command terminal is busy. Try the executing later."));
			gtk_dialog_run(GTK_DIALOG(warning));
			gtk_widget_destroy(warning);
			return FALSE;
		}
	}
	else
	{
		g_spawn_command_line_sync(command,&output,&error,&exit_status,NULL);
		if (exit_status)
		{
			GtkWidget *warning;
			gyum_insert_message(NULL,_("... unsuccessful\n"));
			warning=gtk_message_dialog_new(NULL,
				GTK_DIALOG_MODAL,
				GTK_MESSAGE_WARNING,
				GTK_BUTTONS_OK,
				_("Command executing unsuccessful.\n\nCommand:\n%s\n\nError:\n%s"),
				command,
				error);
			gtk_dialog_run(GTK_DIALOG(warning));
			gtk_widget_destroy(warning);
			return FALSE;
		}
		else
			gyum_insert_message(NULL,_("... successful\n"));
		g_free(error);
		g_free(output);
	}
	return TRUE;
}

void gyum_terminal_stop()
{
	kill(gyumutils_data.pid,SIGTERM);
}

void gyum_insert_message(const gchar *comment,const gchar *msg)
{
	GtkTextBuffer *textbuffer;
	GtkTextIter iter;
	gchar *fullmsg;
	textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(messages));
	gtk_text_buffer_get_end_iter(textbuffer,&iter);
	if (comment)
	{
		fullmsg=g_strdup_printf("%s: %s",comment,msg);
	}
	else
	{
		fullmsg=g_strdup_printf("%s",msg);
	}
	gtk_text_buffer_insert(textbuffer,&iter,fullmsg,-1);
	g_free(fullmsg);
	if (strchr(msg,'\n'))
	{
		GtkTextMark *mark;
		gtk_text_buffer_get_end_iter(textbuffer,&iter);
		mark=gtk_text_buffer_create_mark(textbuffer,NULL,&iter,TRUE);
		gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(messages),mark);
	}
}

void gyum_split_filename(const gchar *filename,gchar **pkgname,gchar **version,gchar **release,gchar **arch)
{
	gchar *str;
	gchar *ptr;
	str=g_strdup(filename);
	ptr=strstr(str,".rpm");
	if (ptr) *ptr=0;
	ptr=strrchr(str,'.');
	if (ptr)
	{
		if (arch) *arch=g_strdup(ptr+1);
		*ptr=0;
	}
	ptr=strrchr(str,'-');
	if (ptr)
	{
		if (release) *release=g_strdup(ptr+1);
		*ptr=0;
	}
	ptr=strrchr(str,'-');
	if (ptr)
	{
		if (version) *version=g_strdup(ptr+1);
		*ptr=0;
	}
	*pkgname=g_strdup(str);
	g_free(str);
}
