#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>
#include "gyum_pkglist.h"
#include "rpmutils.h"
#include "rpminfo.h"
#include "interface.h"
#include "support.h"
#include "gyum_utils.h"
#include "gyum_config.h"

#define CPIO_TEMP_ROOT		"/tmp/gnome-yum"
#define CPIO_TEMP_COMMAND	("/bin/sh "PACKAGE_DATA_DIR"/gnome-yum/gyum-query.sh -f %s%s %s %s")
#define CPIO_EXTRACT_COMMAND	("/bin/sh "PACKAGE_DATA_DIR"/gnome-yum/gyum-query.sh -e %s %s %s %s %s")
#define QUERY_PACKAGE_FILE	"/tmp/query-package.gyum"
#define QUERY_PACKAGE		("/bin/sh "PACKAGE_DATA_DIR"/gnome-yum/gyum-query.sh -pf %s")

void rpminfo_show(const gchar *var,const gchar *val,GtkWidget *window)
{
	gchar *widget_name=g_strdup_printf("rpminfo_%s",var);
	GtkWidget *widget=lookup_widget(window,widget_name);
	if (GTK_IS_ENTRY(widget))
	{
		gtk_entry_set_text(GTK_ENTRY(widget),val);
	}
	else
	if (GNOME_IS_HREF(widget))
	{
		gnome_href_set_text(GNOME_HREF(widget),val);
		gnome_href_set_url(GNOME_HREF(widget),val);
	}
	else
	if (GTK_IS_TEXT_VIEW(widget))
	{
		GtkTextBuffer *textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
		const gchar *str=val;
		while (str && !g_utf8_validate(str,-1,&str))
		{
			if (str)
			{
				*(gchar *)str=' ';
				str++;
			}
		}
		gtk_text_buffer_set_text(textbuffer,val,-1);
	}
	else
	if (GTK_IS_TREE_VIEW(widget))
	{
		GtkListStore *store;
		GtkCellRenderer *renderer;
		GtkTreeViewColumn *column;
		gchar *ptr1;
		gchar *ptr2;
		store=gtk_list_store_new(1,G_TYPE_STRING);
		gtk_tree_view_set_model(GTK_TREE_VIEW(widget),GTK_TREE_MODEL(store));
		renderer=gtk_cell_renderer_text_new();
		column=gtk_tree_view_column_new_with_attributes(_("Name"),renderer,"text",0,NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(widget),column);
		ptr1=(gchar *)val;
		while (ptr1 && *ptr1)
		{
			GtkTreeIter iter;
			ptr2=strchr(ptr1,'\n');
			if (ptr2)
			{
				*ptr2=0;
				ptr2++;
			}
			else
			{
				ptr2=NULL;
			}
			gtk_list_store_append(store,&iter);
			gtk_list_store_set(store,&iter,0,ptr1,-1);
			ptr1=ptr2;
		}
	}
	g_free(widget_name);
}

static GHashTable *rpminfo_open_windows=NULL;
static GHashTable *rpminfo_open_pkgfile=NULL;

void _rpminfo_create(gboolean file,const gchar *name)
{
	if (!rpminfo_open_windows)
		rpminfo_open_windows=g_hash_table_new_full(g_str_hash,g_str_equal,g_free,NULL);
	if (!rpminfo_open_pkgfile)
		rpminfo_open_pkgfile=g_hash_table_new_full(g_str_hash,g_str_equal,g_free,g_free);
	if (name)
	{
		GtkWidget *win;
		win=g_hash_table_lookup(rpminfo_open_windows,name);
		if (!win)
		{
			GHashTable *info;
			GtkWidget *rpminfo=NULL;
			if (file)
				info=rpmutils_pkg_information_from_file(name);
			else
				info=rpmutils_pkg_information(name);
			if (info)
			{
				gchar *title;
				rpminfo=create_gyum_rpminfo();
				g_hash_table_foreach(info,(GHFunc)rpminfo_show,rpminfo);
				title=g_strdup_printf("gnome-yum: %s",name);
				gtk_window_set_title(GTK_WINDOW(rpminfo),title);
				gtk_widget_show(rpminfo);
				g_hash_table_insert(rpminfo_open_windows,g_strdup(name),rpminfo);
				if (file)
				{
					gchar *pkgname;
					pkgname=g_hash_table_lookup(info,"name");
					g_hash_table_insert(rpminfo_open_pkgfile,g_strdup(pkgname),g_strdup(name));
				}
				g_hash_table_destroy(info);
			}
		}
		else
		{
			gtk_window_present(GTK_WINDOW(win));
		}
	}
}

void rpminfo_create(const gchar *pkgname)
{
	_rpminfo_create(FALSE,pkgname);
}

void rpminfo_create_from_file(const gchar *filename)
{
	_rpminfo_create(TRUE,filename);
}

void rpminfo_close(const gchar *pkgname,GtkWidget *window,gpointer data);

void rpminfo_remove(GtkWidget *window)
{
	const gchar *pkgname=NULL;
	pkgname=gtk_window_get_title(GTK_WINDOW(window));
	pkgname+=strlen("gnome-yum: ");
	g_hash_table_remove(rpminfo_open_windows,pkgname);
	rpminfo_close(pkgname,window,NULL);
}

void rpminfo_file_load(pid_t pid,gint status,gchar *uri)
{
	GnomeVFSFileInfo info;
	if (!status && gnome_vfs_get_file_info(uri,
				&info,
				GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==
		GNOME_VFS_OK)
	{
		GtkWidget *gyum_fileview;
		gyum_fileview=create_gyum_fileview();
		if (gyum_fileview)
		{
			const gchar *mime_type;
			GList *apps;
			GList *ai;
			GtkWidget *cbx_select_application;
			GtkWidget *wmime_type;
			gint ret;
			GHashTable *applist;
			GnomeVFSMimeApplication *app;
			GString *command=NULL;
			gchar *text;
			gint i=0;
			gint selected=-1;
			mime_type=gnome_vfs_file_info_get_mime_type(&info);
			applist=g_hash_table_new_full(g_str_hash,
						g_str_equal,
						NULL,
						(GDestroyNotify)gnome_vfs_mime_application_free);
			cbx_select_application=lookup_widget(gyum_fileview,"cbx_select_application");
			wmime_type=lookup_widget(gyum_fileview,"mime_type");
			gtk_label_set_text(GTK_LABEL(wmime_type),mime_type);
			apps=gnome_vfs_mime_get_all_applications(mime_type);
			ai=apps;
			text=gyum_config_get_string(mime_type);
			if (!text)
			{
				GnomeVFSMimeApplication *default_app;
				default_app=gnome_vfs_mime_get_default_application(mime_type);
				if (default_app)
				{
					text=g_strdup(default_app->name);
					gnome_vfs_mime_application_free(default_app);
				}
			}
			while (ai)
			{
				app=ai->data;
				gtk_combo_box_append_text(
					GTK_COMBO_BOX(cbx_select_application),app->name);
				g_hash_table_insert(applist,app->name,app);
				if (text && !strcmp(text,app->name)) selected=i;
				i++;
				ai=ai->next;
			}
			if (selected>-1) gtk_combo_box_set_active(GTK_COMBO_BOX(cbx_select_application),selected);
			if (text) g_free(text);
			g_list_free(apps);
			ret=gtk_dialog_run(GTK_DIALOG(gyum_fileview));
			switch (ret)
			{
				case GTK_RESPONSE_OK:
					text=NULL;
//					if (GTK_MINOR_VERSION==4)
					{
					    GtkTreeModel *model;
					    GtkTreeIter iter;
					    model=gtk_combo_box_get_model(GTK_COMBO_BOX(cbx_select_application));
					    if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(cbx_select_application),&iter))
					    {
						gtk_tree_model_get(model,&iter,0,&text,-1);
					    }
					}
//					else
//					    text=gtk_combo_box_get_active_text(GTK_COMBO_BOX(cbx_select_application));
					if (text && *text)
					{
						app=g_hash_table_lookup(applist,text);
						if (app)
						{
							gyum_config_set_string(mime_type,text);
							command=g_string_new("");
							if (app->requires_terminal)
								g_string_append(command,"gnome-terminal -x ");
							g_string_append_printf(command,"%s %s",app->command,uri);
						}
						g_free(text);
					}
					break;
			}
			gtk_widget_destroy(gyum_fileview);
			g_hash_table_destroy(applist);
			if (command)
			{
				g_spawn_command_line_async(command->str,NULL);
				g_string_free(command,TRUE);
			}
		}
	}
	g_free(uri);
}

gchar *rpminfo_get_selected_file(GtkWidget *rpminfo_files,gchar **packagename)
{
	GtkTreePath *path;
	GtkTreeViewColumn *column;
	GtkTreeIter iter;
	gchar *filename=NULL;
	*packagename=NULL;
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(rpminfo_files),&path,&column);
	if (path)
	{
		GtkTreeModel *model;
		model=gtk_tree_view_get_model(GTK_TREE_VIEW(rpminfo_files));
		gtk_tree_model_get_iter(model,&iter,path);
		gtk_tree_model_get(model,&iter,0,&filename,-1);
		if (filename)
		{
			GtkWidget *gyum_rpminfo;
			gchar *ptr;
			const gchar *title;
			gyum_rpminfo=lookup_widget(rpminfo_files,"gyum_rpminfo");
			title=gtk_window_get_title(GTK_WINDOW(gyum_rpminfo))+strlen("gnome-yum: ");
			if (*title=='/')
			{
				ptr=strstr(title,".rpm");
				if (ptr && !ptr[4]) *packagename=g_strdup(title);
			}
		}
	}
	return filename;
}

void rpminfo_file_open(GtkWidget *widget)
{
	gchar *filename;
	gchar *packagename;
	GtkWidget *rpminfo_files;
	rpminfo_files=lookup_widget(widget,"rpminfo_files");
	filename=rpminfo_get_selected_file(rpminfo_files,&packagename);
	if (filename)
	{
		gchar *uri;
		if (packagename)
		{
			gchar *ptr;
			gchar *command;
			ptr=strrchr(packagename,'/');
			uri=g_strdup_printf("%s%s%s",CPIO_TEMP_ROOT,ptr,filename);
			mkdir(CPIO_TEMP_ROOT,S_IRWXU);
			command=g_strdup_printf(CPIO_TEMP_COMMAND,CPIO_TEMP_ROOT,ptr,packagename,filename);
			gyum_command_execute(_("Extract archive file"),
					command,
					TRUE,
					(GECFunc)rpminfo_file_load,
					uri);
			g_free(packagename);
		}
		else
		{
			uri=g_strdup_printf("%s",filename);
			rpminfo_file_load(0,0,uri);
		}
		g_free(filename);
	}
}

void rpminfo_file_copy(GtkWidget *widget)
{
	GtkWidget *gyum_pkgfile_copy;
	GtkWidget *gyum_pkgfile_copy_dir;
	gchar *dir=NULL;
	gboolean gyum_pkgfile_copy_with_dir=FALSE;
	gchar *dest;
	gyum_pkgfile_copy=create_gyum_pkgfile_copy();
	gyum_pkgfile_copy_dir=lookup_widget(gyum_pkgfile_copy,"gyum_pkgfile_copy_dir");
	dest=gyum_config_get_string("packagefile_dest_dir");
	if (dest)
	{
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(gyum_pkgfile_copy_dir),dest);
		g_free(dest);
	}
	switch (gtk_dialog_run(GTK_DIALOG(gyum_pkgfile_copy)))
	{
		case GTK_RESPONSE_OK:
			dir=gtk_file_chooser_get_current_folder(
					GTK_FILE_CHOOSER(gyum_pkgfile_copy_dir));
			gyum_pkgfile_copy_with_dir=gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(lookup_widget(gyum_pkgfile_copy,"gyum_pkgfile_copy_with_dir")));
			gyum_config_set_string("packagefile_dest_dir",dir);
			break;
	}
	gtk_widget_destroy(gyum_pkgfile_copy);
	if (dir)
	{
		gchar *filename;
		gchar *packagename;
		GtkWidget *rpminfo_files;
		rpminfo_files=lookup_widget(widget,"rpminfo_files");
		filename=rpminfo_get_selected_file(rpminfo_files,&packagename);
		if (filename)
		{
			gchar *command=NULL;
			if (packagename)
			{
				gchar *base;
				gchar *path;
				path=g_path_get_dirname(filename);
				base=g_path_get_basename(filename);
				command=g_strdup_printf(CPIO_EXTRACT_COMMAND,
					gyum_pkgfile_copy_with_dir ? "--with-dir":"--without-dir",
					dir,
					packagename,
					path,
					base);
				g_free(path);
				g_free(base);
			}
			else
			{
				command=g_strdup_printf("cp %s%s %s",
					gyum_pkgfile_copy_with_dir ? "--parents ":"",
					filename,
					dir);
			}
			if (command)
			{
				GError *err=NULL;
				g_spawn_command_line_async(command,&err);
				if (err)
				{
					g_warning("command: %s\n\nerror: %s\n",command,err->message);
					g_error_free(err);
				}
				g_free(command);
			}
		}
	}
}

void rpminfo_clean_filename(gchar *filename)
{
	gchar *ptr;
	ptr=strchr(filename,'(');
	if (ptr) *ptr=0;
	ptr=strchr(filename,'>');
	if (ptr) *ptr=0;
	ptr=strchr(filename,'<');
	if (ptr) *ptr=0;
	ptr=strchr(filename,'=');
	if (ptr) *ptr=0;
	while ((ptr=strchr(filename,' '))!=NULL) *ptr=0;
}

gchar *rpminfo_check_filename(gchar *filename)
{
	if ((*filename!='/') && !yumutils_get_package(filename))
	{
		const gchar **path=NULL;
		if (strstr(filename,"lib")==filename)
		{
			static const gchar *lib_search_path[]={
				"/lib",
				"/usr/lib",
				"/usr/local/lib",
				"/usr/X11R6/lib",NULL};
			path=lib_search_path;
		}
		else
		{
			static const gchar *bin_search_path[]={
				"/bin","/sbin",
				"/usr/bin","/usr/sbin","/usr/libexec",
				"/usr/local/bin","/usr/local/sbin","/usr/local/libexec",
				"/usr/X11R6/bin",NULL};
			path=bin_search_path;
		}
		if (path)
		{
			gint i;
			gchar *new_filename=NULL;
			for (i=0;path[i];i++)
			{
				new_filename=g_strdup_printf("%s/%s",path[i],filename);
				if (!access(new_filename,F_OK))
					break;
				else
				{
					g_free(new_filename);
					new_filename=NULL;
				}
			}
			if (new_filename)
			{
				g_free(filename);
				filename=new_filename;
			}
		}
	}
	return filename;
}

void rpminfo_pkginfo_open(GtkWidget *treeview)
{
	GtkTreePath *path;
	GtkTreeViewColumn *column;
	GtkTreeIter iter;
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(treeview),&path,&column);
	if (path)
	{
		gchar *filename;
		GtkTreeModel *model=gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
		gtk_tree_model_get_iter(model,&iter,path);
		gtk_tree_model_get(model,&iter,0,&filename,-1);
		if (strcmp(filename,_("(none)")))
		{
			rpminfo_clean_filename(filename);
			filename=rpminfo_check_filename(filename);
			if (*filename=='/')
			{
				gchar *cmd;
				cmd=g_strdup_printf(QUERY_PACKAGE,filename);
				if (gyum_command_execute(_("Loading package information"),cmd,FALSE,NULL,NULL))
				{
					FILE *file;
					file=fopen(QUERY_PACKAGE_FILE,"r");
					if (file)
					{
						const gchar *pkgname;
						gchar buffer[1024];
						gulong rb;
						rb=fread(buffer,1,1024,file);
						buffer[rb]=0;
						pkgname=gtk_entry_get_text(GTK_ENTRY(lookup_widget(treeview,"rpminfo_name")));
						if (!strcmp(pkgname,buffer))
						{
							GtkWidget *msg;
							msg=gtk_message_dialog_new(NULL,
									GTK_DIALOG_MODAL,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
									_("'%s' part of this package: %s"),filename,pkgname);
							gtk_dialog_run(GTK_DIALOG(msg));
							gtk_widget_destroy(msg);
						}
						else
						{
							rpminfo_create(buffer);
						}
						fclose(file);
					}
					remove(QUERY_PACKAGE);
				}
				g_free(cmd);
			}
			else
			{
				if (!strchr(filename,'.')) rpminfo_create(filename);
			}
		}
		g_free(filename);
	}
}

void rpminfo_show_requires(GtkWidget *widget)
{
	GtkWidget *rpminfo_requires=lookup_widget(widget,"rpminfo_requires");
	rpminfo_pkginfo_open(rpminfo_requires);
}

void rpminfo_show_conflicts(GtkWidget *widget)
{
	GtkWidget *rpminfo_conflicts=lookup_widget(widget,"rpminfo_conflicts");
	rpminfo_pkginfo_open(rpminfo_conflicts);
}

void rpminfo_show_depends(GtkWidget *widget)
{
	GtkWidget *depends=lookup_widget(widget,"rpminfo_depends");
	GtkTextBuffer *textbuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(depends));
	GtkTextIter start,end;
	gtk_text_buffer_get_start_iter(textbuffer,&start);
	gtk_text_buffer_get_end_iter(textbuffer,&end);
	if (gtk_text_iter_equal(&start,&end))
	{
		const gchar *pkgname;
		pkgname=gtk_entry_get_text(GTK_ENTRY(lookup_widget(widget,"rpminfo_name")));
		if (pkgname)
		{			
			gchar *depends;
			const gchar *filename;
			filename=g_hash_table_lookup(rpminfo_open_pkgfile,pkgname);
			if (filename)
				depends=rpmutils_pkg_depends_from_file(filename);
			else
				depends=rpmutils_pkg_depends(pkgname);
			gtk_text_buffer_set_text(textbuffer,depends ? depends:_("(none)"),-1);
			g_free(depends);
		}
	}
}

void rpminfo_close(const gchar *pkgname,GtkWidget *window,gpointer data)
{
	gchar *ptr=NULL;
	if (*pkgname=='/') ptr=strstr(pkgname,".rpm");
	if (ptr && !ptr[4])
	{
		GtkWidget *rpminfo_name;
		gchar *command;
		GError *err=NULL;
		command=g_strdup_printf("rm -rf %s%s || rm -rf %s",CPIO_TEMP_ROOT,strrchr(pkgname,'/'),CPIO_TEMP_ROOT);
		g_spawn_command_line_async(command,&err);
		if (err)
		{
			g_warning("command: %s\n\nerror:\n%s\n",command,err->message);
			g_error_free(err);
		}
		g_free(command);
		rpminfo_name=lookup_widget(window,"rpminfo_name");
		if (rpminfo_name)
		{
			pkgname=gtk_entry_get_text(GTK_ENTRY(rpminfo_name));
			if (pkgname)
			{
				g_hash_table_remove(rpminfo_open_pkgfile,pkgname);
			}
		}
	}
}

void rpminfo_close_all()
{
	g_hash_table_foreach(rpminfo_open_windows,(GHFunc)rpminfo_close,NULL);
	g_hash_table_destroy(rpminfo_open_windows);
	g_hash_table_destroy(rpminfo_open_pkgfile);
}
