#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <glib/gi18n.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "gyum_utils.h"
#include "rpmutils.h"

#define RPM_QEURY_FILE	"/tmp/query-rpm.gyum"

/*#define CMD_RPM_QUERY "rpm -q --queryformat \"\
name:%%{NAME}\\n\
version:%%{VERSION}\\n\
release:%%{RELEASE}\\n\
group:%%{GROUP}\\n\
install_date:%%{INSTALLTIME}\\n\
summary:%%{SUMMARY}\\n\
license:%%{LICENSE}\\n\
build_host:%%{BUILDHOST}\\n\
build_date:%%{BUILDTIME}\\n\
vendor:%%{VENDOR}\\n\
packager:%%{PACKAGER}\\n\
url:%%{URL}\\n\
size:%%{SIZE}\\n\
signature:pgp=%%{SIGPGP},md5=%%{SIGMD5}\\n\
Mdescription:%%{DESCRIPTION}\\n++---++\\n\
Mconflicts:\" --conflicts --queryformat \"\\n++---++\\n\
Mchangelog:\" --changelog --queryformat \"\\n++---++\\n\
Mrequires:\" -R --queryformat \"\\n++---++\\n\
Mfiles:\" -l\
 %s > /tmp/query-rpm.gyum"*/

#define CMD_RPM_QUERY			("/bin/sh "PACKAGE_DATA_DIR"/gnome-yum/gyum-query.sh -pi %s")
#define RPM_TEST_DEPEND_FILE	"/tmp/query-depend.gyum"
#define RPM_TEST_DEPEND_CMD		"rpm -e --test %s &> /tmp/query-depend.gyum"

void rpmutils_split_line(const gchar *line,gchar **var,gchar **val)
{
	gchar *ptr;
	*var=g_strdup(line);
	*val=NULL;
	ptr=strchr(*var,':');
	if (ptr)
	{
		*ptr=0;
		*val=g_strdup(ptr+1);
		ptr=strchr(*val,'\n');
		if (ptr) *ptr=0;
	}
}

GHashTable *_rpmutils_pkg_information(gboolean file,const gchar *name)
{
	GHashTable *info=NULL;
	gchar *cmd;
	info=g_hash_table_new_full(g_str_hash,g_str_equal,g_free,g_free);
	if (file)
	{
		gchar *str;
		str=g_strdup_printf("-p %s",name);
		cmd=g_strdup_printf(CMD_RPM_QUERY,str);
		g_free(str);
	}
	else
		cmd=g_strdup_printf(CMD_RPM_QUERY,name);
	if (gyum_command_execute(_("Loading information"),cmd,FALSE,NULL,NULL))
	{
		FILE *file=fopen(RPM_QEURY_FILE,"r");
		if (file)
		{
			gchar *var;
			gchar *val;
			gchar *multiline_name=NULL;
			gchar *multiline=NULL;
			gchar *buffer=g_new(gchar,1024);
			while (!feof(file) && !ferror(file) && fgets(buffer,1024,file))
			{
				if (!multiline_name)
				{
					rpmutils_split_line(buffer,&var,&val);
					if (*var=='M')
					{
						multiline_name=g_strdup(var+1);
						multiline=g_strdup_printf("%s\n",val);
						g_free(var);
						g_free(val);
					}
					else
					{
						if (strstr(var,"_date"))
						{
							struct tm *time;
							time_t t=(time_t)atoi(val);
							time=localtime(&t);
							g_free(val);
							val=g_new(gchar,32);
							strftime(val,32,"%x %X",time);
						}
						g_hash_table_insert(info,var,
						    g_locale_to_utf8(val,-1,NULL,NULL,NULL));
						g_free(val);
					}
				}
				else
				{
					if (strcmp(buffer,"++---++\n"))
					{
						gchar *old=multiline;
						multiline=g_strdup_printf("%s%s",old,buffer);
						g_free(old);
					}
					else
					{
						g_hash_table_insert(info,multiline_name,
						    g_locale_to_utf8(multiline,-1,NULL,NULL,NULL));
						g_free(multiline);
						multiline=NULL;
						multiline_name=NULL;
					}
				}
			}
			if (multiline_name)
				g_hash_table_insert(info,multiline_name,multiline);
			g_free(buffer);
			fclose(file);
			remove(RPM_QEURY_FILE);
		}
	}
	g_free(cmd);
	return info;
}

GHashTable *rpmutils_pkg_information(const gchar *name)
{
	return _rpmutils_pkg_information(FALSE,name);
}

GHashTable *rpmutils_pkg_information_from_file(const gchar *name)
{
	return _rpmutils_pkg_information(TRUE,name);
}

gchar *_rpmutils_pkg_depends(gboolean file,const gchar *name)
{
	gchar *depends=NULL;
	gchar *cmd;
	if (file)
	{
		gchar *str;
		str=g_strdup_printf("$(rpm -q --queryformat %%{NAME} -p %s)",name);
		cmd=g_strdup_printf(RPM_TEST_DEPEND_CMD,str);
		g_free(str);
	}
	else
		cmd=g_strdup_printf(RPM_TEST_DEPEND_CMD,name);
	system(cmd);
	{
		FILE *file=fopen(RPM_TEST_DEPEND_FILE,"r");
		if (file)
		{
			gchar *ptr;
			long int length;
			fseek(file,0,SEEK_END);
			length=ftell(file);
			rewind(file);
			depends=g_new(gchar,length+1);
			fread(depends,1,length,file);
			depends[length]=0;
			ptr=strchr(depends,'\n');
			if (ptr) strcpy(depends,ptr+1);
			fclose(file);
			remove(RPM_TEST_DEPEND_FILE);
		}
		else
			g_warning("Depend information file not found...");
	}
	g_free(cmd);
	return depends;
}

gchar *rpmutils_pkg_depends(const gchar *name)
{
	return _rpmutils_pkg_depends(FALSE,name);
}

gchar *rpmutils_pkg_depends_from_file(const gchar *name)
{
	return _rpmutils_pkg_depends(TRUE,name);
}
