#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib/gi18n.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "gyum_utils.h"
#include "gyum_appbar.h"
#include "yumutils.h"

#define INSTALLED_PACKAGES	"/tmp/query-installed.gyum"
#define AVAILABLE_PACKAGES	"/tmp/query-available.gyum"
#define CMD_QUERY_PACKAGES	("/bin/sh " PACKAGE_DATA_DIR "/gnome-yum/gyum-query.sh -a")
#define CMD_QUERY_INSTALLED_PACKAGES	("/bin/sh " PACKAGE_DATA_DIR "/gnome-yum/gyum-query.sh -i")

GHashTable *yum_list=NULL;
GHashTable *yum_package_info_cache=NULL;

void yumutils_init()
{
	remove(INSTALLED_PACKAGES);
	remove(AVAILABLE_PACKAGES);
}

void yumpackage_free(YUMPackage *pkg)
{
	if (pkg)
	{
		int i;
		g_free(pkg->name);
		for (i=0;i<2;i++)
		{
			g_free(pkg->version[i]);
			g_free(pkg->release[i]);
			g_free(pkg->size[i]);
		}
		g_free(pkg->arch);
		g_free(pkg->repository);
		g_free(pkg->summary);
		g_free(pkg->description);
		g_free(pkg->operation);
		g_free(pkg);
	}
}

void yumutils_split_line(gchar *buffer,gchar **var,gchar **val)
{
	gchar *ptr;
	ptr=strchr(buffer,':');
	if (ptr)
	{
		*var=buffer;
		*ptr=0;
		*val=ptr+1;
		ptr--;
		while (*ptr==' ')
		{
			*ptr=0;
			ptr--;
		}
		while (**val==' ') (*val)++;
	}
	else
	{
		*var=NULL;
		*val=buffer;
	}
	ptr=strchr(*val,'\n');
	if (ptr) *ptr=0;
}

typedef enum {
	VERSION_CHANGE_LEVEL_NONE,
	VERSION_CHANGE_LEVEL_MAJOR,
	VERSION_CHANGE_LEVEL_MINOR,
	VERSION_CHANGE_LEVEL_MINI
} VersionChangeLevel;

int rpmvercmp(const char * a, const char * b);

gint yumutils_version_cmp(const gchar *ver1,const gchar *ver2,VersionChangeLevel *l)
{
	gint ch=0;
	*l=VERSION_CHANGE_LEVEL_NONE;
	if ((ch=rpmvercmp(ver1,ver2))<0)
	{
		gchar **ver1v;
		gchar **ver2v;
		gint i;
		ver1v=g_strsplit(ver1,".",0);
		ver2v=g_strsplit(ver2,".",0);
		for (i=0;ver1v[i] && ver2v[i];i++)
		{
			ch=rpmvercmp(ver1v[i],ver2v[i]);
			if (ch) break;
		}
		if (ch<0)
		{
			switch (i)
			{
				case 0:
					*l=VERSION_CHANGE_LEVEL_MAJOR;
					break;
				case 1:
					*l=VERSION_CHANGE_LEVEL_MINOR;
					break;
				default:
					*l=VERSION_CHANGE_LEVEL_MINI;
					break;
			}
		}
		else
		if (!ch)
		{
			if (!ver1v[i] && ver2v[i])
			{
				*l=VERSION_CHANGE_LEVEL_MINI;
				ch=-1;
			}
			else
			if (ver1v[i] && !ver2v[i])
			{
				ch=1;
			}
		}
		g_strfreev(ver1v);
		g_strfreev(ver2v);
	}
	return ch;
}

void yumutils_set_pkg_status(YUMPackage *pkg)
{
	if (!pkg->version[YUM_PACKAGE_STATUS_INSTALLED])
		pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_NOT_INSTALLED;
	else
	if (pkg->version[YUM_PACKAGE_STATUS_AVAILABLE])
	{
		VersionChangeLevel chl;
		gint ch;
		ch=yumutils_version_cmp(pkg->version[YUM_PACKAGE_STATUS_INSTALLED],
								pkg->version[YUM_PACKAGE_STATUS_AVAILABLE],&chl);
		if (ch<=0)
		{
			switch (chl)
			{
				case VERSION_CHANGE_LEVEL_NONE:
					if (pkg->release[YUM_PACKAGE_STATUS_INSTALLED] &&
						pkg->release[YUM_PACKAGE_STATUS_AVAILABLE] &&
						(((ch=rpmvercmp(pkg->release[YUM_PACKAGE_STATUS_INSTALLED],
								pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]))<0)))
					{
						pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_RELEASE_CHANGE;
					}
					else
					{
						g_free(pkg->version[YUM_PACKAGE_STATUS_AVAILABLE]);
						pkg->version[YUM_PACKAGE_STATUS_AVAILABLE]=NULL;
						g_free(pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]);
						pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]=NULL;
						g_free(pkg->size[YUM_PACKAGE_STATUS_AVAILABLE]);
						pkg->size[YUM_PACKAGE_STATUS_AVAILABLE]=NULL;
						g_free(pkg->repository);
						pkg->repository=g_strdup("installed");
						pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_INSTALLED;
					}
					break;
				case VERSION_CHANGE_LEVEL_MAJOR:
					pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_MAJOR_CHANGE;
					break;
				case VERSION_CHANGE_LEVEL_MINOR:
					pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_MINOR_CHANGE;
					break;
				case VERSION_CHANGE_LEVEL_MINI:
					pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_MINI_CHANGE;
					break;
			}
		}
		else
		{
			g_free(pkg->version[YUM_PACKAGE_STATUS_AVAILABLE]);
			pkg->version[YUM_PACKAGE_STATUS_AVAILABLE]=NULL;
			g_free(pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]);
			pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]=NULL;
			g_free(pkg->size[YUM_PACKAGE_STATUS_AVAILABLE]);
			pkg->size[YUM_PACKAGE_STATUS_AVAILABLE]=NULL;
			g_free(pkg->repository);
			pkg->repository=g_strdup("installed");
			pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_INSTALLED;
		}
	}
	else
		pkg->status=(pkg->status&PKG_LIST_FILTER_PROVIDER_MASK)|PKG_LIST_FILTER_INSTALLED;
}

typedef struct {
	YPLFunc func;
} YUMUtilsPackageListData;

void yumutils_package_list_by_type(const gchar *cmd,GECFunc func,gpointer data)
{
	gyum_command_execute(_("Loading packages information"),cmd,TRUE,func,data);
}

void yumutils_package_list_parse_file(const gchar *filename)
{
	FILE *file=fopen(filename,"r");
	if (file)
	{
		gchar buffer[1024];
		YUMPackage *pkg=NULL;
		YUMPackageStatus status=YUM_PACKAGE_STATUS_NONE;
		gchar *description=NULL;
		gchar *var;
		gchar *val;
		while (!feof(file) && !ferror(file) && fgets(buffer,1024,file))
		{
			if (!strcmp(buffer,"Installed Packages\n"))
			{
				status=YUM_PACKAGE_STATUS_INSTALLED;
			}
			else
			if (!strcmp(buffer,"Available Packages\n"))
			{
				status=YUM_PACKAGE_STATUS_AVAILABLE;
			}
			else
			if (status!=YUM_PACKAGE_STATUS_NONE)
			{
				if (strstr(buffer,"Name   :")==buffer)
				{
					if (pkg)
					{
						if (description)
						{
						    pkg->description=g_locale_to_utf8(description,-1,NULL,NULL,NULL);
						    g_free(description);
						}
						yumutils_set_pkg_status(pkg);
					}
					description=NULL;
					yumutils_split_line(buffer,&var,&val);
					pkg=g_hash_table_lookup(yum_list,val);
					if (!pkg)
					{
						pkg=g_new0(YUMPackage,1);
						pkg->name=g_strdup(val);
						pkg->operation=g_strdup("None");
						pkg->status=PKG_LIST_FILTER_DISTRIBUTOR|PKG_LIST_FILTER_OTHER|PKG_LIST_FILTER_NOT_INSTALLED;
						if (strcmp(pkg->name,"gpg-pubkey"))
							g_hash_table_insert(yum_list,g_strdup(pkg->name),pkg);
					}
				}
				else
				{
					if (strstr(buffer,"Version:")==buffer)
					{
						yumutils_split_line(buffer,&var,&val);
						if (!strcmp(pkg->name,"gpg-pubkey"))
						{
							g_free(pkg->name);
							pkg->name=g_strdup_printf("gpg-pubkey-%s",val);
							g_hash_table_insert(yum_list,g_strdup(pkg->name),pkg);
						}
						else
							pkg->version[status]=g_strdup(val);
					}
					else
					if (strstr(buffer,"Release:")==buffer)
					{
						yumutils_split_line(buffer,&var,&val);
						if (strstr(pkg->name,"gpg-pubkey")==pkg->name)
						{
							pkg->version[status]=g_strdup(val);
						}
						else
							pkg->release[status]=g_strdup(val);
					}
					else
					if (strstr(buffer,"Arch   :")==buffer)
					{
						yumutils_split_line(buffer,&var,&val);
						pkg->arch=g_strdup(val);
					}
					else
					if (strstr(buffer,"Size   :")==buffer)
					{
						yumutils_split_line(buffer,&var,&val);
						pkg->size[status]=g_strdup(val);
					}
					else
					if (strstr(buffer,"Repo   :")==buffer)
					{
						yumutils_split_line(buffer,&var,&val);
						if (pkg->repository) g_free(pkg->repository);
						pkg->repository=g_strdup(val);
						if (!strcmp(pkg->repository,"installed"))
							pkg->status=(pkg->status&PKG_LIST_FILTER_STATUS_MASK)|PKG_LIST_FILTER_DISTRIBUTOR|PKG_LIST_FILTER_OTHER;
						else
						if (!strcmp(pkg->repository,"base") ||
							!strcmp(pkg->repository,"updates") ||
							!strcmp(pkg->repository,"updates-released"))
						{
							pkg->status=(pkg->status&PKG_LIST_FILTER_STATUS_MASK)|PKG_LIST_FILTER_DISTRIBUTOR;
						}
						else
						{
							pkg->status=(pkg->status&PKG_LIST_FILTER_STATUS_MASK)|PKG_LIST_FILTER_OTHER;
						}
					}
					else
					if (strstr(buffer,"Summary:")==buffer)
					{
						if (!pkg->summary) 
						{
							yumutils_split_line(buffer,&var,&val);
							pkg->summary=g_locale_to_utf8(val,-1,NULL,NULL,NULL);
						}
					}
					else
					if (strstr(buffer,"Description:")==buffer)
					{
						if (!pkg->description) description=g_strdup("");
					}
					else
					if (description)
					{
						gchar *old=description;
						description=g_strdup_printf("%s%s",description,buffer);
						g_free(old);
					}
				}
			}
		}
		if (pkg)
		{
			if (description)
			{
			    pkg->description=g_locale_to_utf8(description,-1,NULL,NULL,NULL);
			    g_free(description);
			}
			yumutils_set_pkg_status(pkg);
		}
		fclose(file);
	}
}

void yumutils_package_list_parse()
{
	yumutils_package_list_parse_file(INSTALLED_PACKAGES);
	yumutils_package_list_parse_file(AVAILABLE_PACKAGES);
}

void yumutils_package_list_check(const gchar *id,YUMPackage *package,GHashTable *new_list)
{
	if (*package->operation!='N')
	{
		YUMPackage *nli;
		nli=g_hash_table_lookup(new_list,id);
		if (nli)
		{
			if (!strcmp(package->operation,"Install") && 
				((nli->status&PKG_LIST_FILTER_STATUS_MASK)<=PKG_LIST_FILTER_NOT_INSTALLED))
			{
				g_free(nli->operation);
				nli->operation=g_strdup("Install");
				return;
			}
			if (!strcmp(package->operation,"Update") &&
				((nli->status&PKG_LIST_FILTER_STATUS_MASK)>=PKG_LIST_FILTER_UPDATES))
			{
				g_free(nli->operation);
				nli->operation=g_strdup("Update");
				return;
			}
			if (!strcmp(package->operation,"Remove") &&
				((nli->status&PKG_LIST_FILTER_STATUS_MASK)>=PKG_LIST_FILTER_INSTALLED))
			{
				g_free(nli->operation);
				nli->operation=g_strdup("Remove");
				return;
			}
		}
	}
}

void yumutils_package_list_done(pid_t pid,gint status,YPLFunc func)
{
	GHashTable *old_list=yum_list;
	GHashTable *new_list=NULL;
	if (!yum_list && status) remove(AVAILABLE_PACKAGES);
	new_list=
			g_hash_table_new_full(g_str_hash,g_str_equal,g_free,(GDestroyNotify)yumpackage_free);
	if (new_list)
	{
		yum_list=new_list;
		yumutils_package_list_parse();
		if (old_list)
		{
			g_hash_table_foreach(old_list,(GHFunc)yumutils_package_list_check,yum_list);
			g_hash_table_destroy(old_list);
		}
	}
	remove(INSTALLED_PACKAGES);
	if (func) func(status);
}

void yumutils_package_list_clean()
{
	remove(AVAILABLE_PACKAGES);
	remove(INSTALLED_PACKAGES);
}

void yumutils_package_list(YPLFunc func)
{
	yumutils_package_list_by_type(CMD_QUERY_PACKAGES,(GECFunc)yumutils_package_list_done,func);
}

void yumutils_package_list_installed(YPLFunc func)
{
	yumutils_package_list_by_type(CMD_QUERY_INSTALLED_PACKAGES,(GECFunc)yumutils_package_list_done,func);
}

gint yumutils_size()
{
	return yum_list ? g_hash_table_size(yum_list):0;
}

void yumutils_foreach(YPFunc func,gpointer data)
{
	g_hash_table_foreach(yum_list,(GHFunc)func,data);
}

void yumutils_free()
{
	if (yum_list) g_hash_table_destroy(yum_list);
	remove(INSTALLED_PACKAGES);
	remove(AVAILABLE_PACKAGES);
}

gboolean yumutils_package_list_available()
{
	return yum_list!=NULL;
}

const YUMPackage *yumutils_get_package(const gchar *name)
{
	const YUMPackage *pkg=yum_list ? g_hash_table_lookup(yum_list,name):NULL;
	return pkg;
}

GHashTable *yumutils_pkg_information(const gchar *name)
{
	GHashTable *info=NULL;
	const YUMPackage *pkg;
	pkg=yumutils_get_package(name);
	if (pkg)
	{
		info=g_hash_table_new_full(g_str_hash,g_str_equal,g_free,NULL);
		g_hash_table_insert(info,g_strdup("name"),pkg->name);
		g_hash_table_insert(info,g_strdup("version"),
			pkg->version[YUM_PACKAGE_STATUS_AVAILABLE]);
		g_hash_table_insert(info,g_strdup("release"),
			pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]);
		g_hash_table_insert(info,g_strdup("summary"),
			pkg->summary);
		g_hash_table_insert(info,g_strdup("description"),
			pkg->description);
	}
	return info;
}

gboolean yumutils_file_exists(const gchar *filename)
{
	gboolean exists;
	gchar *path;
	path=g_strdup_printf("%s/.gnome-yum/%s",g_getenv("HOME"),filename);
	exists=!access(path,F_OK);
	g_free(path);
	return exists;
}

void yumutils_file_save_one(const gchar *name,YUMPackage *pkg,FILE *file)
{
	if (strcmp(pkg->operation,"None"))
	{
		fprintf(file,"%s:%s(%s-%s=>%s-%s)\n",
			name,pkg->operation,
			pkg->version[YUM_PACKAGE_STATUS_INSTALLED] ? pkg->version[YUM_PACKAGE_STATUS_INSTALLED]:"0",
			pkg->release[YUM_PACKAGE_STATUS_INSTALLED] ? pkg->release[YUM_PACKAGE_STATUS_INSTALLED]:"0",
			pkg->version[YUM_PACKAGE_STATUS_AVAILABLE],pkg->release[YUM_PACKAGE_STATUS_AVAILABLE]);
	}
}

void yumutils_file_save(const gchar *filename)
{
	FILE *file;
	gchar *path;
	gchar *dir;
	dir=g_strdup_printf("%s/.gnome-yum",g_getenv("HOME"));
	mkdir(dir,S_IRWXU);
	path=g_strdup_printf("%s/%s",dir,filename);
	file=fopen(path,"w+");
	if (file)
	{
		fprintf(file,"# This is the gnome-yum selection status file, do not modify anything...\n");
		yumutils_foreach((YPFunc)yumutils_file_save_one,file);
		fclose(file);
	}
	g_free(path);
	g_free(dir);
}

gboolean yumutils_file_load(const gchar *filename)
{
	FILE *file;
	gchar *path;
	gboolean loading_successful=FALSE;
	path=g_strdup_printf("%s/.gnome-yum/%s",g_getenv("HOME"),filename);
	file=fopen(path,"r");
	if (file)
	{
		gchar line[1024];
		while (!feof(file) && !ferror(file) && fgets(line,1024,file))
		{
			gchar *name;
			gchar *operation;
			operation=strchr(line,':');
			if (operation)
			{
				gchar *ptr;
				name=line;
				*operation=0;
				operation++;
				ptr=strchr(operation,'(');
				if (ptr)
				{
					YUMPackage *pkg;
					*ptr=0;
					pkg=(YUMPackage *)yumutils_get_package(name);
					if (!strcmp(operation,"Install") && 
						((pkg->status&PKG_LIST_FILTER_STATUS_MASK)<=PKG_LIST_FILTER_NOT_INSTALLED))
					{
						g_free(pkg->operation);
						pkg->operation=g_strdup("Install");
						continue;
					}
					if (!strcmp(operation,"Update") &&
						((pkg->status&PKG_LIST_FILTER_STATUS_MASK)>=PKG_LIST_FILTER_UPDATES))
					{
						g_free(pkg->operation);
						pkg->operation=g_strdup("Update");
						continue;
					}
					if (!strcmp(operation,"Remove") &&
						((pkg->status&PKG_LIST_FILTER_STATUS_MASK)>=PKG_LIST_FILTER_INSTALLED))
					{
						g_free(pkg->operation);
						pkg->operation=g_strdup("Remove");
						continue;
					}
				}
			}
		}
		loading_successful=!ferror(file);
		fclose(file);
	}
	g_free(path);
	return loading_successful;
}

void yumutils_file_remove(const gchar *filename)
{
	gchar *path;
	path=g_strdup_printf("%s/.gnome-yum/%s",g_getenv("HOME"),filename);
	if (remove(path))
		g_warning("%s: %s",strerror(errno),path);
	g_free(path);
}
