/***************************************************************************
 *            yumutils.h
 *
 *  Tue Mar  8 16:51:46 2005
 *  Copyright  2005  Tóth, András
 *  toth_bandi@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _YUMUTILS_H
#define _YUMUTILS_H

#include <glib.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum {
	YUM_PACKAGE_STATUS_NONE=-1,
	YUM_PACKAGE_STATUS_AVAILABLE=0,
	YUM_PACKAGE_STATUS_INSTALLED=1
} YUMPackageStatus;

typedef enum {
	PKG_LIST_FILTER_ALL=0,
	PKG_LIST_FILTER_NOT_INSTALLED=1,
	PKG_LIST_FILTER_INSTALLED=2,
	PKG_LIST_FILTER_UPDATES=3,
	PKG_LIST_FILTER_RELEASE_CHANGE=4,
	PKG_LIST_FILTER_MINI_CHANGE=5,
	PKG_LIST_FILTER_MINOR_CHANGE=6,
	PKG_LIST_FILTER_MAJOR_CHANGE=7,
	PKG_LIST_FILTER_STATUS_MASK=7,
	// provider settings
	PKG_LIST_FILTER_DISTRIBUTOR=8,
	PKG_LIST_FILTER_OTHER=16,
	PKG_LIST_FILTER_PROVIDER_MASK=24
} PkgListFilter;

typedef struct {
	gchar *name;
	gchar *arch;
	gchar *version[2];
	gchar *release[2];
	gchar *size[2];
	gchar *repository;
	gchar *summary;
	gchar *description;
	gchar *operation;
	PkgListFilter status;
} YUMPackage;

typedef void (*YPFunc)(const gchar *,const YUMPackage *,gpointer);
typedef void (*YPLFunc)(gint);

void yumutils_init();
gboolean yumutils_package_list_available();
void yumutils_package_list_clean();
void yumutils_package_list(YPLFunc func);
void yumutils_package_list_installed(YPLFunc func);
gint yumutils_size();
void yumutils_foreach(YPFunc func,gpointer data);
void yumutils_free();
const YUMPackage *yumutils_get_package(const gchar *name);
GHashTable *yumutils_pkg_information(const gchar *name);
gboolean yumutils_file_exists(const gchar *filename);
void yumutils_file_save(const gchar *filename);
gboolean yumutils_file_load(const gchar *filename);
void yumutils_file_remove(const gchar *filename);

#ifdef __cplusplus
}
#endif

#endif /* _YUMUTILS_H */
